@@ -19,13 +19,17 @@ from .deploy import init, shutdown
 
 
 def _install():
+    from .lightgbm import _install as _install_lightgbm
     from .numpy import _install as _install_numpy
     from .pandas import _install as _install_pandas
     from .web import _install as _install_web
+    from .xgboost import _install as _install_xgboost
 
     _install_pandas()
     _install_numpy()
     _install_web()
+    _install_lightgbm()
+    _install_xgboost()
 
 
 _install()
