@@ -1,6 +1,5 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
-# Copyright 1999-2021 Alibaba Group Holding Ltd.
+# Copyright 2022-2023 XProbe Inc.
+# derived from copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,11 +13,14 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from . import _version
 from .config import options
 from .core.context import get_context
-from .session import new_session, execute, fetch, fetch_log, stop_server
-from .deploy.oscar import new_cluster_in_ray, new_ray_session
+from .core.entrypoints import init_extension_entrypoints
+from .session import execute, fetch, fetch_log, new_session, stop_server
 
-from . import _version
+# load third party extensions.
+init_extension_entrypoints()
+del init_extension_entrypoints
 
 __version__ = _version.get_versions()["version"]
