@@ -1,4 +1,4 @@
-# Copyright 2022 XProbe Inc.
+# Copyright 2022-2023 XProbe Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,9 +12,17 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-# Env key for Mars log absolute path
-MARS_LOG_PATH_KEY = "MARS_LOG_PATH"
-# Mars log file name prefix
-MARS_LOG_PREFIX = "mars_"
-# The prefix of the temporary directory where the Mars log is located
-MARS_TMP_DIR_PREFIX = "mars_tmp"
+import os
+from pathlib import Path
+
+MARS_TEMP_DIR = str(Path.home() / ".xorbits")
+
+# logging.
+MARS_LOG_DIR_KEY = "MARS_LOG_DIR"
+DEFAULT_MARS_LOG_DIR = os.path.join(MARS_TEMP_DIR, "logs")
+DEFAULT_MARS_LOG_FILE_NAME = "xorbits.log"
+DEFAULT_MARS_LOG_MAX_BYTES = 100 * 1024 * 1024
+DEFAULT_MARS_LOG_BACKUP_COUNT = 30
+
+# profiling.
+MARS_PROFILING_RESULTS_DIR = os.path.join(MARS_TEMP_DIR, "profiling")
