@@ -1,4 +1,5 @@
-# Copyright 1999-2021 Alibaba Group Holding Ltd.
+# Copyright 2022-2023 XProbe Inc.
+# derived from copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,37 +13,37 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import itertools
 import copy
+import itertools
 from functools import reduce
 
 import numpy as np
 import pandas as pd
 
-from ...core import ENTITY_TYPE, CHUNK_TYPE, recursive_tile
+from ...core import CHUNK_TYPE, ENTITY_TYPE, recursive_tile
 from ...serialization.serializables import AnyField
-from ...tensor.core import TENSOR_TYPE, TENSOR_CHUNK_TYPE, ChunkData, Chunk
+from ...tensor.core import TENSOR_CHUNK_TYPE, TENSOR_TYPE, Chunk, ChunkData
 from ...utils import classproperty, get_dtype
 from ..align import (
-    align_series_series,
-    align_dataframe_series,
     align_dataframe_dataframe,
+    align_dataframe_series,
+    align_series_series,
 )
 from ..core import (
-    DATAFRAME_TYPE,
-    SERIES_TYPE,
     DATAFRAME_CHUNK_TYPE,
+    DATAFRAME_TYPE,
     SERIES_CHUNK_TYPE,
+    SERIES_TYPE,
     is_chunk_meta_lazy,
 )
-from ..operands import DataFrameOperandMixin, DataFrameOperand
+from ..operands import DataFrameOperand, DataFrameOperandMixin
 from ..ufunc.tensor import TensorUfuncMixin
 from ..utils import (
-    parse_index,
-    infer_dtypes,
-    infer_dtype,
-    infer_index_value,
     build_empty_df,
+    infer_dtype,
+    infer_dtypes,
+    infer_index_value,
+    parse_index,
 )
 
 
@@ -212,6 +213,7 @@ class DataFrameBinOpMixin(DataFrameOperandMixin):
         for chunk in tileable.chunks:
             out_op = op.copy().reset_key()
             if chunk.ndim == 2:
+                dtypes = df.dtypes[chunk.dtypes.index]
                 if lazy_chunk_meta:
                     out_chunk = out_op.new_chunk(
                         [chunk],
@@ -223,14 +225,14 @@ class DataFrameBinOpMixin(DataFrameOperandMixin):
                         nsplits=tileable.nsplits,
                         index_value=df.index_value,
                         columns_value=df.columns_value,
-                        dtypes=df.dtypes,
+                        dtypes=dtypes,
                     )
                 else:
                     out_chunk = out_op.new_chunk(
                         [chunk],
                         shape=chunk.shape,
                         index=chunk.index,
-                        dtypes=chunk.dtypes,
+                        dtypes=dtypes,
                         index_value=chunk.index_value,
                         columns_value=getattr(chunk, "columns_value"),
                     )
@@ -240,7 +242,7 @@ class DataFrameBinOpMixin(DataFrameOperandMixin):
                         [chunk],
                         shape=chunk.shape,
                         index=chunk.index,
-                        dtype=chunk.dtype,
+                        dtype=df.dtype,
                         name=getattr(chunk, "name"),
                     )
                     out_chunk._set_tileable_meta(
@@ -253,7 +255,7 @@ class DataFrameBinOpMixin(DataFrameOperandMixin):
                         [chunk],
                         shape=chunk.shape,
                         index=chunk.index,
-                        dtype=chunk.dtype,
+                        dtype=df.dtype,
                         index_value=chunk.index_value,
                         name=getattr(chunk, "name"),
                     )
