@@ -1,6 +1,5 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
-# Copyright 1999-2021 Alibaba Group Holding Ltd.
+# Copyright 2022-2023 XProbe Inc.
+# derived from copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,12 +12,12 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
 from io import BytesIO
+from typing import Any, Dict, List, Tuple, Union
 from urllib.parse import urlparse
 
-import pandas as pd
 import numpy as np
+import pandas as pd
 
 try:
     from pyarrow import NativeFile
@@ -28,31 +27,31 @@ except ImportError:  # pragma: no cover
 from ... import opcodes as OperandDef
 from ...config import options
 from ...core import OutputType
-from ...lib.filesystem import get_fs, open_file, file_size, glob
+from ...lib.filesystem import file_size, get_fs, glob, open_file
 from ...serialization.serializables import (
-    StringField,
+    AnyField,
+    BoolField,
     DictField,
-    ListField,
     Int32Field,
     Int64Field,
-    BoolField,
-    AnyField,
+    ListField,
+    StringField,
 )
-from ...utils import parse_readable_size, lazy_import, FixedSizeFileObject
+from ...utils import FixedSizeFileObject, lazy_import, parse_readable_size
 from ..arrays import ArrowStringDtype
-from ..utils import parse_index, build_empty_df, to_arrow_dtypes, contain_arrow_dtype
+from ..utils import build_empty_df, contain_arrow_dtype, parse_index, to_arrow_dtypes
 from .core import (
-    IncrementalIndexDatasource,
     ColumnPruneSupportedDataSourceMixin,
+    IncrementalIndexDatasource,
     IncrementalIndexDataSourceMixin,
     merge_small_files,
 )
-
+from .utils import convert_to_abspath
 
 cudf = lazy_import("cudf")
 
 
-def _find_delimiter(f, block_size=2**16):
+def _find_delimiter(f: Any, block_size: int = 2**16):
     delimiter = b"\n"
     if f.tell() == 0:
         return 0
@@ -64,7 +63,7 @@ def _find_delimiter(f, block_size=2**16):
             return f.tell() - len(b) + b.index(delimiter) + 1
 
 
-def _find_hdfs_start_end(f, offset, size):
+def _find_hdfs_start_end(f: Any, offset: int, size: int):
     # As pyarrow doesn't support `readline` operation (https://github.com/apache/arrow/issues/3838),
     # we need to find the start and end of file block manually.
 
@@ -78,12 +77,12 @@ def _find_hdfs_start_end(f, offset, size):
     return start, end
 
 
-def _find_chunk_start_end(f, offset, size):
+def _find_chunk_start_end(f: Any, offset: int, size: int, is_first: bool):
     if NativeFile is not None and isinstance(f, NativeFile):
         return _find_hdfs_start_end(f, offset, size)
     f.seek(offset)
-    if f.tell() == 0:
-        start = 0
+    if is_first:
+        start = offset
     else:
         f.readline()
         start = f.tell()
@@ -105,6 +104,7 @@ class DataFrameReadCSV(
     sep = StringField("sep")
     header = AnyField("header")
     index_col = Int32Field("index_col")
+    skiprows = Int32Field("skiprows")
     compression = StringField("compression")
     usecols = AnyField("usecols")
     offset = Int64Field("offset")
@@ -189,8 +189,20 @@ class DataFrameReadCSV(
         for path in paths:
             path = path_prefix + path
             total_bytes = file_size(path, storage_options=op.storage_options)
-            offset = 0
-            for _ in range(int(np.ceil(total_bytes * 1.0 / chunk_bytes))):
+            start_pos = 0
+            if op.skiprows is not None:
+                with open_file(
+                    op.path,
+                    compression=op.compression,
+                    storage_options=op.storage_options,
+                ) as f:
+                    for i in range(op.skiprows):
+                        f.readline()
+                    offset = f.tell()
+                    start_pos = offset
+            else:
+                offset = 0
+            for _ in range(int(np.ceil((total_bytes - start_pos) * 1.0 / chunk_bytes))):
                 chunk_op = op.copy().reset_key()
                 chunk_op.path = path
                 chunk_op.offset = offset
@@ -228,7 +240,7 @@ class DataFrameReadCSV(
     def _pandas_read_csv(cls, f, op):
         csv_kwargs = op.extra_params.copy()
         out_df = op.outputs[0]
-        start, end = _find_chunk_start_end(f, op.offset, op.size)
+        start, end = _find_chunk_start_end(f, op.offset, op.size, out_df.index[0] == 0)
         f.seek(start)
         b = FixedSizeFileObject(f, end - start)
         if hasattr(out_df, "dtypes"):
@@ -243,7 +255,7 @@ class DataFrameReadCSV(
                 # convert to Series, if usecols is a scalar
                 df = df[op.usecols]
         else:
-            if start == 0:
+            if out_df.index[0] == 0 or start == 0:
                 # The first chunk contains header
                 # As we specify names and dtype, we need to skip header rows
                 csv_kwargs["header"] = op.header
@@ -366,19 +378,20 @@ class DataFrameReadCSV(
 
 
 def read_csv(
-    path,
-    names=None,
+    path: str,
+    names: Union[List, Tuple] = None,
     sep: str = ",",
-    index_col=None,
-    compression=None,
-    header="infer",
-    dtype=None,
-    usecols=None,
-    nrows=None,
-    chunk_bytes="64M",
-    gpu=None,
-    head_bytes="100k",
-    head_lines=None,
+    index_col: int = None,
+    compression: str = None,
+    header: Union[str, List] = "infer",
+    dtype: Union[str, Dict] = None,
+    usecols: List = None,
+    nrows: int = None,
+    skiprows: int = None,
+    chunk_bytes: Union[str, int] = "64M",
+    gpu: bool = None,
+    head_bytes: Union[str, int] = "100k",
+    head_lines: int = None,
     incremental_index: bool = True,
     use_arrow_dtype: bool = None,
     storage_options: dict = None,
@@ -482,12 +495,8 @@ def read_csv(
         Values to consider as False.
     skipinitialspace : bool, default False
         Skip spaces after delimiter.
-    skiprows : list-like, int or callable, optional
-        Line numbers to skip (0-indexed) or number of lines to skip (int)
-        at the start of the file.
-        If callable, the callable function will be evaluated against the row
-        indices, returning True if the row should be skipped and False otherwise.
-        An example of a valid callable argument would be ``lambda x: x in [0, 2]``.
+    skiprows : int
+        Number of lines to skip (int) at the start of the file.
     skipfooter : int, default 0
         Number of lines at bottom of file to skip (Unsupported with engine='c').
     nrows : int, optional
@@ -677,6 +686,8 @@ def read_csv(
     >>> # read from OSS
     >>> auth_path = build_oss_path(file_path, access_key_id, access_key_secret, end_point)
     >>> md.read_csv(auth_path)
+    >>> # read from S3
+    >>> md.read_csv('s3://bucket/file.txt')
     """
     # infer dtypes and columns
     if isinstance(path, (list, tuple)):
@@ -690,7 +701,6 @@ def read_csv(
             file_path = glob(path.rstrip("/") + "/*", storage_options)[0]
     else:
         file_path = glob(path, storage_options)[0]
-
     with open_file(
         file_path, compression=compression, storage_options=storage_options
     ) as f:
@@ -698,7 +708,7 @@ def read_csv(
             b = b"".join([f.readline() for _ in range(head_lines)])
         else:
             head_bytes = int(parse_readable_size(head_bytes)[0])
-            head_start, head_end = _find_chunk_start_end(f, 0, head_bytes)
+            head_start, head_end = _find_chunk_start_end(f, 0, head_bytes, True)
             f.seek(head_start)
             b = f.read(head_end - head_start)
         mini_df = pd.read_csv(
@@ -708,6 +718,7 @@ def read_csv(
             dtype=dtype,
             names=names,
             header=header,
+            skiprows=skiprows,
         )
         if header == "infer" and names is not None:
             # ignore header as we always specify names
@@ -729,13 +740,18 @@ def read_csv(
     columns_value = parse_index(mini_df.columns, store_data=True)
     if index_col and not isinstance(index_col, int):
         index_col = list(mini_df.columns).index(index_col)
+
+    # convert path to abs_path
+    abs_path = convert_to_abspath(path, storage_options)
+
     op = DataFrameReadCSV(
-        path=path,
+        path=abs_path,
         names=names,
         sep=sep,
         header=header,
         index_col=index_col,
         usecols=usecols,
+        skiprows=skiprows,
         compression=compression,
         gpu=gpu,
         incremental_index=incremental_index,
