@@ -1,6 +1,5 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
-# Copyright 1999-2021 Alibaba Group Holding Ltd.
+# Copyright 2022-2023 XProbe Inc.
+# derived from copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,8 +14,7 @@
 # limitations under the License.
 
 import os
-import pickle
-from typing import List, Tuple
+from typing import Dict, Optional, Union
 from urllib.parse import urlparse
 
 import numpy as np
@@ -25,8 +23,11 @@ import pandas as pd
 try:
     import pyarrow as pa
     import pyarrow.parquet as pq
+
+    arrow_dtype = pa.DataType
 except ImportError:
     pa = None
+    arrow_dtype = None
 
 try:
     import fastparquet
@@ -35,34 +36,30 @@ except ImportError:
 
 from ... import opcodes as OperandDef
 from ...config import options
-from ...lib.filesystem import file_size, get_fs, glob, open_file, FileSystem
+from ...lib.filesystem import FileSystem, file_size, get_fs, glob, open_file
 from ...serialization.serializables import (
     AnyField,
     BoolField,
     DictField,
-    ListField,
-    StringField,
     Int32Field,
     Int64Field,
-    BytesField,
+    ListField,
+    StringField,
 )
 from ...utils import is_object_dtype, lazy_import
-from ..arrays import ArrowStringDtype
+from ..arrays import ArrowListDtype, ArrowStringDtype
 from ..operands import OutputType
-from ..utils import (
-    parse_index,
-    to_arrow_dtypes,
-    contain_arrow_dtype,
-)
+from ..utils import contain_arrow_dtype, parse_index, to_arrow_dtypes
 from .core import (
-    IncrementalIndexDatasource,
     ColumnPruneSupportedDataSourceMixin,
+    IncrementalIndexDatasource,
     IncrementalIndexDataSourceMixin,
     merge_small_files,
 )
+from .utils import convert_to_abspath
 
-
-PARQUET_MEMORY_SCALE = 15
+PARQUET_MEMORY_SCALE = 10
+PARQUET_MEMORY_SCALE_WITH_ARROW_DTYPE = 3
 STRING_FIELD_OVERHEAD = 50
 cudf = lazy_import("cudf")
 
@@ -116,8 +113,8 @@ class ParquetEngine:
     def read_partitioned_to_pandas(
         self,
         f,
-        partitions: "pq.ParquetPartitions",
-        partition_keys: List[Tuple],
+        partitions: Dict,
+        partition_keys: Dict,
         columns=None,
         nrows=None,
         use_arrow_dtype=None,
@@ -126,11 +123,10 @@ class ParquetEngine:
         raw_df = self.read_to_pandas(
             f, columns=columns, nrows=nrows, use_arrow_dtype=use_arrow_dtype, **kwargs
         )
-        for i, (name, index) in enumerate(partition_keys):
-            dictionary = partitions[i].dictionary
-            value = dictionary[index]
-            raw_df[name] = pd.Series(
-                value.as_py(),
+        for col, value in partition_keys.items():
+            dictionary = partitions[col]
+            raw_df[col] = pd.Series(
+                value,
                 dtype=pd.CategoricalDtype(categories=dictionary.tolist()),
                 index=raw_df.index,
             )
@@ -174,21 +170,35 @@ def _parse_prefix(path):
     return path_prefix
 
 
+def _arrow_dtype_mapper(
+    tp: Union[np.dtype, arrow_dtype]
+) -> Optional[Union[ArrowListDtype, ArrowStringDtype]]:
+    if tp == pa.string():
+        return ArrowStringDtype()
+    elif isinstance(tp, pa.ListType):
+        return ArrowListDtype(tp.value_type)
+    else:
+        return
+
+
 class ArrowEngine(ParquetEngine):
     def get_row_num(self, f):
         file = pq.ParquetFile(f)
         return file.metadata.num_rows
 
     def read_dtypes(self, f, **kwargs):
+        types_mapper = kwargs.pop("types_mapper", None)
         file = pq.ParquetFile(f)
-        return file.schema_arrow.empty_table().to_pandas().dtypes
+        return (
+            file.schema_arrow.empty_table().to_pandas(types_mapper=types_mapper).dtypes
+        )
 
     @classmethod
     def _table_to_pandas(cls, t, nrows=None, use_arrow_dtype=None):
         if nrows is not None:
             t = t.slice(0, nrows)
         if use_arrow_dtype:
-            df = t.to_pandas(types_mapper={pa.string(): ArrowStringDtype()}.get)
+            df = t.to_pandas(types_mapper=_arrow_dtype_mapper)
         else:
             df = t.to_pandas()
         return df
@@ -249,8 +259,8 @@ class CudfEngine:
     def read_partitioned_to_cudf(
         cls,
         file,
-        partitions: "pq.ParquetPartitions",
-        partition_keys: List[Tuple],
+        partitions: Dict,
+        partition_keys: Dict,
         columns=None,
         nrows=None,
         **kwargs,
@@ -262,11 +272,13 @@ class CudfEngine:
         t = t.slice(0, nrows) if nrows is not None else t
         t = pa.table(t.columns, names=t.column_names)
         raw_df = cudf.DataFrame.from_arrow(t)
-        for i, (name, index) in enumerate(partition_keys):
-            dictionary = partitions[i].dictionary
-            codes = cudf.core.column.as_column(index, length=len(raw_df))
-            raw_df[name] = cudf.core.column.build_categorical_column(
-                categories=dictionary.tolist(),
+        for col, value in partition_keys.items():
+            dictionary = partitions[col].tolist()
+            codes = cudf.core.column.as_column(
+                dictionary.index(value), length=len(raw_df)
+            )
+            raw_df[col] = cudf.core.column.build_categorical_column(
+                categories=dictionary,
                 codes=codes,
                 size=codes.size,
                 offset=codes.offset,
@@ -295,8 +307,8 @@ class DataFrameReadParquet(
     merge_small_files = BoolField("merge_small_files")
     merge_small_file_options = DictField("merge_small_file_options")
     # for chunk
-    partitions = BytesField("partitions", default=None)
-    partition_keys = ListField("partition_keys", default=None)
+    partitions = DictField("partitions", default=None)
+    partition_keys = DictField("partition_keys", default=None)
     num_group_rows = Int64Field("num_group_rows", default=None)
     # as read meta may be too time-consuming when number of files is large,
     # thus we only read first file to get row number and raw file size
@@ -325,21 +337,30 @@ class DataFrameReadParquet(
         out_df = op.outputs[0]
         shape = (np.nan, out_df.shape[1])
         dtypes = cls._to_arrow_dtypes(out_df.dtypes, op)
-        dataset = pq.ParquetDataset(op.path)
+        dataset = pq.ParquetDataset(op.path, use_legacy_dataset=False)
 
         path_prefix = _parse_prefix(op.path)
 
         chunk_index = 0
         out_chunks = []
         first_chunk_row_num, first_chunk_raw_bytes = None, None
-        for i, piece in enumerate(dataset.pieces):
+        for i, fragment in enumerate(dataset.fragments):
             chunk_op = op.copy().reset_key()
-            chunk_op.path = chunk_path = path_prefix + piece.path
-            chunk_op.partitions = pickle.dumps(dataset.partitions)
-            chunk_op.partition_keys = piece.partition_keys
+            chunk_op.path = chunk_path = path_prefix + fragment.path
+            relpath = os.path.relpath(chunk_path, op.path)
+            partition_keys = dict(
+                tuple(s.split("=")) for s in relpath.split(os.sep)[:-1]
+            )
+            chunk_op.partition_keys = partition_keys
+            chunk_op.partitions = dict(
+                zip(
+                    dataset.partitioning.schema.names, dataset.partitioning.dictionaries
+                )
+            )
             if i == 0:
-                first_chunk_raw_bytes = file_size(chunk_path, op.storage_options)
-                first_chunk_row_num = piece.get_metadata().num_rows
+                first_row_group = fragment.row_groups[0]
+                first_chunk_raw_bytes = first_row_group.total_byte_size
+                first_chunk_row_num = first_row_group.num_rows
             chunk_op.first_chunk_row_num = first_chunk_row_num
             chunk_op.first_chunk_raw_bytes = first_chunk_raw_bytes
             new_chunk = chunk_op.new_chunk(
@@ -458,11 +479,10 @@ class DataFrameReadParquet(
     def _execute_partitioned(cls, ctx, op: "DataFrameReadParquet"):
         out = op.outputs[0]
         engine = get_engine(op.engine)
-        partitions = pickle.loads(op.partitions)
         with open_file(op.path, storage_options=op.storage_options) as f:
             ctx[out.key] = engine.read_partitioned_to_pandas(
                 f,
-                partitions,
+                op.partitions,
                 op.partition_keys,
                 columns=op.columns,
                 nrows=op.nrows,
@@ -516,10 +536,9 @@ class DataFrameReadParquet(
 
         try:
             if op.partitions is not None:
-                partitions = pickle.loads(op.partitions)
                 ctx[out.key] = engine.read_partitioned_to_cudf(
                     file,
-                    partitions,
+                    op.partitions,
                     op.partition_keys,
                     columns=op.columns,
                     nrows=op.nrows,
@@ -567,10 +586,29 @@ class DataFrameReadParquet(
             .astype(np.int64)
             .item()
         )
-        phy_size = raw_bytes * (op.memory_scale or PARQUET_MEMORY_SCALE)
-        n_strings = len([dt for dt in op.outputs[0].dtypes if is_object_dtype(dt)])
-        pd_size = phy_size + n_strings * estimated_row_num * STRING_FIELD_OVERHEAD
-        ctx[op.outputs[0].key] = (pd_size, pd_size + phy_size)
+        if op.columns is not None:
+            with open_file(op.path, storage_options=op.storage_options) as f:
+                all_columns = list(get_engine(op.engine).read_dtypes(f).index)
+        else:
+            all_columns = list(op.outputs[0].dtypes.index)
+        columns = op.columns if op.columns else all_columns
+        if op.use_arrow_dtype:
+            scale = op.memory_scale or PARQUET_MEMORY_SCALE_WITH_ARROW_DTYPE
+        else:
+            scale = op.memory_scale or PARQUET_MEMORY_SCALE
+        phy_size = raw_bytes * scale * len(columns) / len(all_columns)
+        n_strings = len(
+            [
+                dt
+                for col, dt in op.outputs[0].dtypes.items()
+                if col in columns and is_object_dtype(dt)
+            ]
+        )
+        if op.use_arrow_dtype:
+            pd_size = phy_size
+        else:
+            pd_size = phy_size + n_strings * estimated_row_num * STRING_FIELD_OVERHEAD
+        ctx[op.outputs[0].key] = (pd_size, pd_size)
 
     def __call__(self, index_value=None, columns_value=None, dtypes=None):
         self._output_types = [OutputType.dataframe]
@@ -650,6 +688,14 @@ def read_parquet(
     single_path = path[0] if isinstance(path, list) else path
     fs = get_fs(single_path, storage_options)
     is_partitioned = False
+    if use_arrow_dtype is None:
+        use_arrow_dtype = options.dataframe.use_arrow_dtype
+    if use_arrow_dtype and engine_type != "pyarrow":
+        raise ValueError(
+            f"The 'use_arrow_dtype' argument is not supported for the {engine_type} engine"
+        )
+    types_mapper = _arrow_dtype_mapper if use_arrow_dtype else None
+
     if fs.isdir(single_path):
         paths = fs.ls(path)
         if all(fs.isdir(p) for p in paths):
@@ -658,7 +704,7 @@ def read_parquet(
             is_partitioned = True
         else:
             with fs.open(paths[0], mode="rb") as f:
-                dtypes = engine.read_dtypes(f)
+                dtypes = engine.read_dtypes(f, types_mapper=types_mapper)
     else:
         if not isinstance(path, list):
             file_path = glob(path, storage_options=storage_options)[0]
@@ -666,20 +712,18 @@ def read_parquet(
             file_path = path[0]
 
         with open_file(file_path, storage_options=storage_options) as f:
-            dtypes = engine.read_dtypes(f)
+            dtypes = engine.read_dtypes(f, types_mapper=types_mapper)
 
     if columns:
         dtypes = dtypes[columns]
 
-    if use_arrow_dtype is None:
-        use_arrow_dtype = options.dataframe.use_arrow_dtype
-    if use_arrow_dtype:
-        dtypes = to_arrow_dtypes(dtypes)
-
     index_value = parse_index(pd.RangeIndex(-1))
     columns_value = parse_index(dtypes.index, store_data=True)
+
+    # convert path to abs_path
+    abs_path = convert_to_abspath(path, storage_options)
     op = DataFrameReadParquet(
-        path=path,
+        path=abs_path,
         engine=engine_type,
         columns=columns,
         groups_as_chunks=groups_as_chunks,
