@@ -1,4 +1,5 @@
-# Copyright 1999-2021 Alibaba Group Holding Ltd.
+# Copyright 2022-2023 XProbe Inc.
+# derived from copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,30 +19,30 @@ import pickle
 import uuid
 from typing import List, Union
 
+import cloudpickle
 import numpy as np
 import pandas as pd
-import cloudpickle
 
 from ... import opcodes as OperandDef
 from ...config import options
 from ...core.context import Context
 from ...core.operand import OperatorLogicKeyGeneratorMixin
 from ...serialization.serializables import (
-    StringField,
     AnyField,
     BoolField,
-    ListField,
-    Int64Field,
-    Float64Field,
     BytesField,
+    Float64Field,
+    Int64Field,
+    ListField,
+    StringField,
 )
 from ...tensor.utils import normalize_chunk_sizes
 from ...typing import OperandType, TileableType
 from ..arrays import ArrowStringDtype
-from ..utils import parse_index, create_sa_connection, to_arrow_dtypes
+from ..utils import create_sa_connection, parse_index, to_arrow_dtypes
 from .core import (
-    IncrementalIndexDatasource,
     ColumnPruneSupportedDataSourceMixin,
+    IncrementalIndexDatasource,
     IncrementalIndexDataSourceMixin,
 )
 
@@ -127,7 +128,7 @@ class DataFrameReadSQL(
                     selectable = sa.Table(
                         self.table_or_sql,
                         m,
-                        autoload=True,
+                        autoload_replace=True,
                         autoload_with=engine_or_conn,
                         schema=self.schema,
                     )
@@ -141,12 +142,13 @@ class DataFrameReadSQL(
                             .alias(temp_name_2)
                         )
                     else:
-                        selectable = sql.select(
-                            "*",
-                            from_obj=sql.text(
-                                f"({self.table_or_sql}) AS {temp_name_1}"
-                            ),
-                        ).alias(temp_name_2)
+                        selectable = (
+                            sql.select("*")
+                            .select_from(
+                                sql.text(f"({self.table_or_sql}) AS {temp_name_1}")
+                            )
+                            .alias(temp_name_2)
+                        )
                     self.selectable = selectable
         return selectable
 
@@ -155,11 +157,15 @@ class DataFrameReadSQL(
 
         # fetch test DataFrame
         if columns:
-            query = sql.select(
-                [sql.column(c) for c in columns], from_obj=selectable
-            ).limit(test_rows)
+            query = (
+                sql.select(*[sql.column(c) for c in columns])
+                .select_from(selectable)
+                .limit(test_rows)
+            )
         else:
-            query = sql.select(selectable.columns, from_obj=selectable).limit(test_rows)
+            query = (
+                sql.select(selectable.columns).select_from(selectable).limit(test_rows)
+            )
         test_df = pd.read_sql(
             query,
             engine_or_conn,
@@ -178,7 +184,7 @@ class DataFrameReadSQL(
             # fetch size
             size = list(
                 engine_or_conn.execute(
-                    sql.select([sql.func.count()]).select_from(selectable)
+                    sql.select(sql.func.count()).select_from(selectable)
                 )
             )[0][0]
             shape = (size, test_df.shape[1])
@@ -351,14 +357,15 @@ class DataFrameReadSQL(
             engine = sa.create_engine(op.con, **(op.engine_kwargs or dict()))
             try:
                 part_col = selectable.columns[op.partition_col]
-                range_results = engine.execute(
-                    sql.select([sql.func.min(part_col), sql.func.max(part_col)])
-                )
+                with engine.connect() as connection:
+                    range_results = connection.execute(
+                        sql.select(*[sql.func.min(part_col), sql.func.max(part_col)])
+                    )
 
-                op.low_limit, op.high_limit = next(range_results)
-                if op.parse_dates and op.partition_col in op.parse_dates:
-                    op.low_limit = op._parse_datetime(op.low_limit)
-                    op.high_limit = op._parse_datetime(op.high_limit)
+                    op.low_limit, op.high_limit = next(range_results)
+                    if op.parse_dates and op.partition_col in op.parse_dates:
+                        op.low_limit = op._parse_datetime(op.low_limit)
+                        op.high_limit = op._parse_datetime(op.high_limit)
             finally:
                 engine.dispose()
 
@@ -444,7 +451,7 @@ class DataFrameReadSQL(
             op.low_limit = _adapt_datetime(op.low_limit)
             op.high_limit = _adapt_datetime(op.high_limit)
 
-            query = sa.sql.select(columns)
+            query = sa.sql.select(*columns)
             if op.method == "partition":
                 part_col = selectable.columns[op.partition_col]
                 if op.left_end:
@@ -497,8 +504,10 @@ class DataFrameReadSQL(
                 for i in range(len(dtypes)):
                     dtype = dtypes.iloc[i]
                     if isinstance(dtype, ArrowStringDtype):
-                        df.iloc[:, i] = df.iloc[:, i].astype(dtype)
-
+                        if pd.__version__ >= "1.5.0":
+                            df.isetitem(i, df.iloc[:, i].astype(dtype))
+                        else:  # pragma: no cover
+                            df.iloc[:, i] = df.iloc[:, i].astype(dtype)
             if out.ndim == 2:
                 ctx[out.key] = df
             else:
