@@ -1,4 +1,5 @@
-# Copyright 1999-2021 Alibaba Group Holding Ltd.
+# Copyright 2022-2023 XProbe Inc.
+# derived from copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,7 +17,7 @@ import functools
 import itertools
 import logging
 import uuid
-from typing import Callable, Dict, List, Union
+from typing import Callable, Dict, List, Optional, Union
 
 import numpy as np
 import pandas as pd
@@ -24,47 +25,43 @@ import pandas as pd
 from ... import opcodes as OperandDef
 from ...config import options
 from ...core import ENTITY_TYPE, OutputType
-from ...core.custom_log import redirect_custom_log
 from ...core.context import get_context
+from ...core.custom_log import redirect_custom_log
 from ...core.operand import OperandStage
 from ...serialization.serializables import (
-    Int32Field,
-    Int64Field,
     AnyField,
     BoolField,
-    StringField,
-    ListField,
     DictField,
+    Int32Field,
+    Int64Field,
+    ListField,
+    StringField,
 )
 from ...typing import ChunkType, TileableType
 from ...utils import (
     enter_current_session,
+    estimate_pandas_size,
     lazy_import,
     pd_release_version,
-    estimate_pandas_size,
 )
 from ..arrays import ArrowArray
 from ..core import GROUPBY_TYPE
 from ..merge import DataFrameConcat
 from ..operands import DataFrameOperand, DataFrameOperandMixin, DataFrameShuffleProxy
-from ..reduction.core import (
-    ReductionCompiler,
-    ReductionSteps,
-    ReductionAggStep,
-)
 from ..reduction.aggregation import is_funcs_aggregate, normalize_reduction_funcs
+from ..reduction.core import ReductionAggStep, ReductionCompiler, ReductionSteps
 from ..utils import (
-    parse_index,
     build_concatenated_rows_frame,
-    is_cudf,
     concat_on_columns,
+    is_cudf,
+    parse_index,
 )
 from .core import DataFrameGroupByOperand
 from .custom_aggregation import custom_agg_functions
 from .sort import (
-    DataFramePSRSGroupbySample,
     DataFrameGroupbyConcatPivot,
     DataFrameGroupbySortShuffle,
+    DataFramePSRSGroupbySample,
 )
 
 cp = lazy_import("cupy", rename="cp")
@@ -235,7 +232,14 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
         )
 
         shape = (np.nan, agg_df.shape[1])
-        index_value = parse_index(agg_df.index, groupby.key, groupby.index_value.key)
+        if isinstance(agg_df.index, pd.RangeIndex):
+            index_value = parse_index(
+                pd.RangeIndex(-1), groupby.key, groupby.index_value.key
+            )
+        else:
+            index_value = parse_index(
+                agg_df.index, groupby.key, groupby.index_value.key
+            )
 
         # make sure if as_index=False takes effect
         self._fix_as_index(agg_df.index)
@@ -483,9 +487,7 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
                 by = []
                 for v in map_op.groupby_params["by"]:
                     if isinstance(v, ENTITY_TYPE):
-                        by_chunk = v.cix[
-                            chunk.index[0],
-                        ]
+                        by_chunk = v.cix[chunk.index[0],]
                         chunk_inputs.append(by_chunk)
                         by.append(by_chunk)
                     else:
@@ -562,7 +564,6 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
         sample_chunks: List[ChunkType],
         agg_chunk_len: int,
     ):
-
         properties = dict(
             by=op.groupby_params["by"],
             gpu=op.is_gpu(),
@@ -1005,7 +1006,13 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
             return custom_agg_functions[func_name].execute_agg(op, in_data)
 
     @staticmethod
-    def _do_predefined_agg(input_obj, agg_func, single_func=False, **kwds):
+    def _do_predefined_agg(
+        input_obj,
+        agg_func,
+        single_func: bool = False,
+        gpu: Optional[bool] = False,
+        **kwds,
+    ):
         ndim = getattr(input_obj, "ndim", None) or input_obj.obj.ndim
         if agg_func == "str_concat":
             agg_func = lambda x: x.str.cat(**kwds)
@@ -1016,10 +1023,14 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
 
         if ndim == 2:
             if single_func:
-                result = input_obj.agg(agg_func)
-                if result.ndim == 1:
-                    # when agg_func == size, agg only returns one single series.
-                    result = result.to_frame(agg_func)
+                # DataFrameGroupby.agg('size') returns empty df in cudf, which is not correct
+                # The index order of .size() is wrong in cudf,
+                # however, for performance considerations, sort_index() will not be called here
+                result = (
+                    input_obj.size()
+                    if gpu and agg_func == "size"
+                    else input_obj.agg(agg_func)
+                )
             else:
                 result = input_obj.agg([agg_func])
                 result.columns = result.columns.droplevel(-1)
@@ -1109,7 +1120,7 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
                 single_func = map_func_name == op.raw_func
                 agg_dfs.append(
                     cls._do_predefined_agg(
-                        input_obj, map_func_name, single_func, **kwds
+                        input_obj, map_func_name, single_func, op.gpu, **kwds
                     )
                 )
 
@@ -1155,7 +1166,7 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
                 combines.append(cls._do_custom_agg(raw_func_name, op, raw_input))
             else:
                 combines.append(
-                    cls._do_predefined_agg(input_obj, agg_func_name, **kwds)
+                    cls._do_predefined_agg(input_obj, agg_func_name, gpu=op.gpu, **kwds)
                 )
         ctx[op.outputs[0].key] = tuple(combines)
 
@@ -1198,7 +1209,7 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
             else:
                 input_obj = cls._get_grouped(op, in_data_dict[output_key], ctx)
                 in_data_dict[output_key] = cls._do_predefined_agg(
-                    input_obj, agg_func_name, **kwds
+                    input_obj, agg_func_name, gpu=op.gpu, **kwds
                 )
 
         aggs = []
@@ -1248,7 +1259,14 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
                 result.reset_index(
                     inplace=True, drop=result.index.name in result.columns
                 )
-            result = result.reindex(col_value, axis=1)
+            if isinstance(col_value, xdf.MultiIndex) and not col_value.is_unique:
+                # reindex doesn't work when the agg function list contains duplicated
+                # functions, e.g. df.groupby(...)agg((func, func))
+                result = result.iloc[:, result.columns.duplicated()]
+                result = xdf.concat([result[c] for c in col_value], axis=1)
+                result.columns = col_value
+            else:
+                result = result.reindex(col_value, axis=1)
 
             if result.ndim == 2 and len(result) == 0:
                 result = result.astype(out_chunk.dtypes)
