@@ -1,4 +1,5 @@
-# Copyright 1999-2021 Alibaba Group Holding Ltd.
+# Copyright 2022-2023 XProbe Inc.
+# derived from copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,23 +21,22 @@ import pandas as pd
 
 from ... import opcodes as OperandDef
 from ...core import ENTITY_TYPE, Entity, OutputType
-from ...core.operand import OperandStage, MapReduceOperand
+from ...core.operand import MapReduceOperand, OperandStage
 from ...lib.groupby_wrapper import wrapped_groupby
-from ...serialization.serializables import BoolField, Int32Field, AnyField
-from ...utils import lazy_import, pd_release_version, no_default
+from ...serialization.serializables import AnyField, BoolField, Int32Field
+from ...utils import lazy_import, no_default, pd_release_version
 from ..align import align_dataframe_series, align_series_series
+from ..core import SERIES_CHUNK_TYPE, SERIES_TYPE
 from ..initializer import Series as asseries
-from ..core import SERIES_TYPE, SERIES_CHUNK_TYPE
+from ..operands import DataFrameOperandMixin, DataFrameShuffleProxy
 from ..utils import (
     build_concatenated_rows_frame,
-    hash_dataframe_on,
     build_df,
     build_series,
-    parse_index,
+    hash_dataframe_on,
     is_cudf,
+    parse_index,
 )
-from ..operands import DataFrameOperandMixin, DataFrameShuffleProxy
-
 
 cudf = lazy_import("cudf")
 
@@ -48,6 +48,7 @@ class DataFrameGroupByOperand(MapReduceOperand, DataFrameOperandMixin):
     _op_type_ = OperandDef.GROUPBY
 
     _by = AnyField("by", on_serialize=lambda x: x.data if isinstance(x, Entity) else x)
+    _is_list_by = BoolField("is_list_by")
     _level = AnyField("level")
     _as_index = BoolField("as_index")
     _sort = BoolField("sort")
@@ -62,6 +63,7 @@ class DataFrameGroupByOperand(MapReduceOperand, DataFrameOperandMixin):
         as_index=None,
         sort=None,
         group_keys=None,
+        is_list_by=None,
         shuffle_size=None,
         output_types=None,
         **kw
@@ -72,6 +74,7 @@ class DataFrameGroupByOperand(MapReduceOperand, DataFrameOperandMixin):
             _as_index=as_index,
             _sort=sort,
             _group_keys=group_keys,
+            _is_list_by=is_list_by,
             _shuffle_size=shuffle_size,
             _output_types=output_types,
             **kw
@@ -115,6 +118,10 @@ class DataFrameGroupByOperand(MapReduceOperand, DataFrameOperandMixin):
     def group_keys(self):
         return self._group_keys
 
+    @property
+    def is_list_by(self):
+        return self._is_list_by
+
     @property
     def shuffle_size(self):
         return self._shuffle_size
@@ -277,9 +284,7 @@ class DataFrameGroupByOperand(MapReduceOperand, DataFrameOperandMixin):
                 chunk_by = []
                 for k in by:
                     if isinstance(k, SERIES_TYPE):
-                        by_chunk = k.cix[
-                            chunk.index[0],
-                        ]
+                        by_chunk = k.cix[chunk.index[0],]
                         chunk_by.append(by_chunk)
                         chunk_inputs.append(by_chunk)
                     else:
@@ -471,7 +476,7 @@ class DataFrameGroupByOperand(MapReduceOperand, DataFrameOperandMixin):
                         else:
                             by[i] = v
         else:
-            r = pd.concat(res, axis=0)
+            r = xdf.concat(res, axis=0)
 
         if chunk.index_value is not None:
             if isinstance(r, tuple):
@@ -500,13 +505,24 @@ class DataFrameGroupByOperand(MapReduceOperand, DataFrameOperandMixin):
             else:
                 df = inp
                 by = op.by
+            if not op.is_list_by and isinstance(by, list):
+                by = by[0]
             ctx[op.outputs[0].key] = wrapped_groupby(
                 df,
                 by=by,
                 level=op.level,
                 as_index=op.as_index,
                 sort=op.sort,
-                group_keys=op.group_keys if op.group_keys is not None else no_default,
+                # For GPU, unexpected problems would happen if ``group_keys=no_default``:
+                # ValueError: Length mismatch: Expected axis has 4 elements, new values have 8 elements
+                # from stack:
+                # cudf/core/groupby/groupby.py:784: in apply
+                #     result.index = cudf.MultiIndex._from_data(index_data)
+                # cudf/core/dataframe.py:1108: in __setattr__
+                #     super().__setattr__(key, col)
+                group_keys=op.group_keys
+                if op.group_keys is not None or op.is_gpu() is True
+                else no_default,
             )
 
 
@@ -519,6 +535,7 @@ def groupby(
     output_types = (
         [OutputType.dataframe_groupby] if df.ndim == 2 else [OutputType.series_groupby]
     )
+    is_list_by = isinstance(by, (list, tuple))
     if isinstance(by, (SERIES_TYPE, pd.Series)):
         if isinstance(by, pd.Series):
             by = asseries(by)
@@ -531,6 +548,7 @@ def groupby(
         as_index=as_index,
         sort=sort,
         group_keys=group_keys if group_keys is not no_default else None,
+        is_list_by=is_list_by,
         output_types=output_types,
     )
     return op(df)
