@@ -1,4 +1,5 @@
-# Copyright 1999-2021 Alibaba Group Holding Ltd.
+# Copyright 2022-2023 XProbe Inc.
+# derived from copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,18 +22,17 @@ from pandas.core.dtypes.cast import find_common_type
 from pandas.core.indexing import IndexingError
 
 from ... import opcodes as OperandDef
-from ...core import ENTITY_TYPE, OutputType, recursive_tile
 from ...config import options
+from ...core import ENTITY_TYPE, OutputType, recursive_tile
 from ...serialization.serializables import AnyField, KeyField, ListField
 from ...tensor import asarray
 from ...tensor.datasource.empty import empty
 from ...tensor.indexing.core import calc_shape
 from ...utils import ceildiv
-from ..operands import DataFrameOperand, DataFrameOperandMixin, DATAFRAME_TYPE
+from ..operands import DATAFRAME_TYPE, DataFrameOperand, DataFrameOperandMixin
 from ..utils import indexing_index_value, is_cudf
 from .index_lib import DataFrameIlocIndexesHandler
 
-
 _ILOC_ERROR_MSG = (
     "Location based indexing can only have [integer, "
     "integer slice (START point is INCLUDED, END point is EXCLUDED), "
@@ -541,7 +541,7 @@ class SeriesIlocGetItem(DataFrameOperand, HeadTailOptimizedOperandMixin):
         return [(yield from handler.handle(op))]
 
     @classmethod
-    def execute(cls, ctx, op):
+    def execute(cls, ctx, op: "SeriesIlocGetItem"):
         chunk = op.outputs[0]
         series = ctx[op.input.key]
         if len(op.inputs) > 1:
@@ -551,6 +551,8 @@ class SeriesIlocGetItem(DataFrameOperand, HeadTailOptimizedOperandMixin):
             )
         else:
             indexes = tuple(op.indexes)
+        if op.is_gpu() and len(indexes) == 1:
+            indexes = indexes[0]
         if hasattr(series, "iloc"):
             ctx[chunk.key] = series.iloc[indexes]
         else:
