@@ -1,4 +1,5 @@
-# Copyright 1999-2021 Alibaba Group Holding Ltd.
+# Copyright 2022-2023 XProbe Inc.
+# derived from copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,21 +25,21 @@ from ... import opcodes
 from ...core import ENTITY_TYPE, recursive_tile
 from ...core.operand import OperandStage
 from ...serialization.serializables import (
-    KeyField,
     AnyField,
-    StringField,
-    Int64Field,
     BoolField,
+    Int64Field,
+    KeyField,
+    StringField,
 )
 from ...tensor import tensor as astensor
 from ...utils import lazy_import, pd_release_version
-from ..core import Index as DataFrameIndexType, INDEX_TYPE
+from ..core import INDEX_TYPE
+from ..core import Index as DataFrameIndexType
 from ..initializer import Index as asindex
 from ..operands import DataFrameOperand, DataFrameOperandMixin
-from ..utils import validate_axis_style_args, parse_index
+from ..utils import parse_index, validate_axis_style_args
 from .index_lib import DataFrameReindexHandler
 
-
 cudf = lazy_import("cudf")
 
 # under pandas<1.1, SparseArray ignores zeros on creation
@@ -331,7 +332,7 @@ class DataFrameReindex(DataFrameOperand, DataFrameOperandMixin):
         inp = cls._convert_to_writable(ctx[op.input.key])
         index = cls._get_value(ctx, op.index)
         if op.index_freq is not None:
-            index = pd.Index(index, freq=op.index_freq)
+            index = type(pd.Index(index))(index, freq=op.index_freq)
         columns = cls._get_value(ctx, op.columns)
         kw = {"level": op.level}
         if index is not None and not isinstance(index, slice):
