@@ -1,4 +1,5 @@
-# Copyright 1999-2021 Alibaba Group Holding Ltd.
+# Copyright 2022-2023 XProbe Inc.
+# derived from copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,32 +22,33 @@ import numpy as np
 import pandas as pd
 
 from ...core import (
-    OutputType,
     ENTITY_TYPE,
+    OutputType,
+    enter_mode,
     is_build_mode,
     is_kernel_mode,
-    enter_mode,
     recursive_tile,
 )
 from ...core.operand import OperandStage
 from ...serialization.serializables import (
-    BoolField,
     AnyField,
+    BoolField,
     DataTypeField,
     Int32Field,
     StringField,
 )
 from ...utils import pd_release_version, tokenize
 from ..core import SERIES_TYPE
+from ..operands import DATAFRAME_TYPE, DataFrameOperand, DataFrameOperandMixin
 from ..utils import (
-    parse_index,
     build_df,
     build_empty_df,
-    build_series,
     build_empty_series,
+    build_series,
+    is_cudf,
+    parse_index,
     validate_axis,
 )
-from ..operands import DataFrameOperandMixin, DataFrameOperand, DATAFRAME_TYPE
 
 # in pandas<1.3, when aggregating with multiple levels and numeric_only is True,
 # object cols not ignored with min-max funcs
@@ -300,6 +302,7 @@ class DataFrameReductionMixin(DataFrameOperandMixin):
                 _output_type=output_type,
                 _dtypes=dtypes,
                 _index=index,
+                _func_name=getattr(op, "_func_name"),
             )
         )
         return [out_df]
@@ -331,7 +334,11 @@ class DataFrameReductionMixin(DataFrameOperandMixin):
         elif func_name == "custom_reduction":
             empty_df = build_df(df, ensure_string=True)
             reduced = getattr(self, "custom_reduction").__call_agg__(empty_df)
-            reduced_cols = list(reduced.index)
+            reduced_cols = (
+                list(reduced.index.to_pandas())
+                if is_cudf(reduced)
+                else list(reduced.index)
+            )
             reduced_dtype = reduced.dtype
         else:
             reduced_cols, dtypes = [], []
@@ -874,7 +881,7 @@ class ReductionCompiler:
             self._update_col_dict(self._output_key_to_post_cols, step.output_key, cols)
 
     def _compile_expr_function(self, py_src: str, local_consts: dict):
-        from ... import tensor, dataframe
+        from ... import dataframe, tensor
 
         result_store = dict()
         global_vars = globals().copy()
@@ -889,15 +896,24 @@ class ReductionCompiler:
         return fun
 
     @staticmethod
-    def _build_mock_return_object(func, input_dtype, ndim):
-        from ..initializer import DataFrame as MarsDataFrame, Series as MarsSeries
+    def _build_mock_return_object(func, func_idx: int, input_dtype, ndim):
+        from ..initializer import DataFrame as MarsDataFrame
+        from ..initializer import Series as MarsSeries
+
+        # for identical functions, the output objects will have the same keys, which may cause
+        # incorrect results.
+        # the index could be an ideal way to identify each of the functions.
+        func_name = f"f_{func_idx}"
 
         if ndim == 1:
-            mock_series = build_empty_series(np.dtype(input_dtype))
+            mock_series = build_empty_series(np.dtype(input_dtype), name=func_name)
             mock_obj = MarsSeries(mock_series)
         else:
             mock_df = build_empty_df(
-                pd.Series([np.dtype(input_dtype)] * 2, index=["A", "B"])
+                pd.Series(
+                    [np.dtype(input_dtype)] * 2,
+                    index=[f"{func_name}_A", f"{func_name}_B"],
+                )
             )
             mock_obj = MarsDataFrame(mock_df)
 
@@ -911,10 +927,11 @@ class ReductionCompiler:
         from ...tensor.base import TensorWhere
         from ..arithmetic.core import DataFrameBinOp, DataFrameUnaryOp
         from ..datasource.dataframe import DataFrameDataSource
-        from ..indexing.where import DataFrameWhere
         from ..datasource.series import SeriesDataSource
+        from ..indexing.where import DataFrameWhere
 
-        func_token = tokenize(func, self._axis, func_name, ndim)
+        func_idx = len(self._compiled_funcs)
+        func_token = tokenize(func, self._axis, func_name, func_idx, ndim)
         if func_token in _func_compile_cache:
             return _func_compile_cache[func_token]
         custom_reduction = func if isinstance(func, CustomReduction) else None
@@ -922,10 +939,10 @@ class ReductionCompiler:
         self._check_function_valid(func)
 
         try:
-            func_ret = self._build_mock_return_object(func, float, ndim=ndim)
-        except (TypeError, AttributeError):
+            func_ret = self._build_mock_return_object(func, func_idx, float, ndim=ndim)
+        except (TypeError, AttributeError):  # pragma: no cover
             # we may encounter lambda x: x.str.cat(...), use an object series to test
-            func_ret = self._build_mock_return_object(func, object, ndim=1)
+            func_ret = self._build_mock_return_object(func, func_idx, object, ndim=1)
         output_limit = getattr(func, "output_limit", None) or 1
 
         if not isinstance(func_ret, ENTITY_TYPE):
@@ -1147,6 +1164,24 @@ class ReductionCompiler:
                     axis_expr = f"axis={op_axis!r}, " if op_axis is not None else ""
                     op_str = _func_name_to_op[func_name]
                     if t.op.lhs is t.inputs[0]:
+                        if (
+                            (
+                                func_name
+                                in (
+                                    "gt",
+                                    "ge",
+                                    "lt",
+                                    "le",
+                                    "eq",
+                                    "ne",
+                                )
+                            )
+                            and isinstance(t.op.lhs, DATAFRAME_TYPE)
+                            and isinstance(t.op.rhs, str)
+                        ):
+                            # for a cudf dataframe, df == 'foo' doesn't work, so we convert the rhs
+                            # to a tuple.
+                            rhs = f"({rhs},) * len({lhs}.columns)"
                         statements = [
                             f"try:",
                             f"    {var_name} = {lhs}.{func_name}({rhs}, {axis_expr})",
