@@ -1,4 +1,5 @@
-# Copyright 1999-2021 Alibaba Group Holding Ltd.
+# Copyright 2022-2023 XProbe Inc.
+# derived from copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,39 +13,35 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import os
-import sys
-import cloudpickle
 import functools
 import itertools
-import inspect
 import logging
 import operator
 from contextlib import contextmanager
 from numbers import Integral
-from typing import List, Union, Any
+from typing import Any, Callable, List, Optional, Union
 
 import numpy as np
 import pandas as pd
-from pandas.api.types import is_string_dtype
 from pandas.api.extensions import ExtensionDtype
+from pandas.api.types import is_string_dtype
 from pandas.core.dtypes.cast import find_common_type
 
 from ..config import options
 from ..core import Entity, ExecutableTuple
-from ..core.context import Context, get_context
+from ..core.context import Context
 from ..lib.mmh3 import hash as mmh_hash
+from ..storage import StorageLevel
+from ..tensor.array_utils import is_cupy
 from ..tensor.utils import dictify_chunk_size, normalize_chunk_sizes
 from ..typing import ChunkType, TileableType
 from ..utils import (
-    tokenize,
-    sbytes,
-    lazy_import,
     ModulePlaceholder,
     is_full_slice,
+    lazy_import,
     parse_readable_size,
-    is_on_ray,
-    parse_version,
+    sbytes,
+    tokenize,
 )
 
 try:
@@ -54,14 +51,6 @@ except ImportError:  # pragma: no cover
 
 cudf = lazy_import("cudf", rename="cudf")
 vineyard = lazy_import("vineyard")
-try:
-    import ray
-
-    ray_release_version = parse_version(ray.__version__).release
-    ray_deprecate_ml_dataset = ray_release_version[:2] >= (2, 0)
-except ImportError:
-    ray_release_version = None
-    ray_deprecate_ml_dataset = None
 logger = logging.getLogger(__name__)
 
 
@@ -74,31 +63,93 @@ def hash_index(index, size):
     return [idx_to_grouped.get(i, list()) for i in range(size)]
 
 
+def _get_hash(
+    data: Union[
+        pd.DataFrame, pd.Series, pd.Index, "cudf.DataFrame", "cudf.Series", "cudf.Index"
+    ],
+    **kwargs,
+) -> Union[pd.Series, "cudf.Series"]:
+    """
+    The hash value of the cudf object is obtained by the ``hash_values`` interface.
+    Specifically, for the Index obj in cudf, convert it to DataFrame first.
+    """
+    return (
+        (data.to_frame().hash_values() if is_index(data) else data.hash_values())
+        if is_cudf(data)
+        else pd.util.hash_pandas_object(data, **kwargs)
+    )
+
+
+def _get_index_for_on(
+    data: Union[pd.Series, "cudf.Series"], size: int
+) -> List[Union[pd.Index, "cudf.Index"]]:
+    if is_cudf(data):
+        # Since cudf does not support groupby for RangeIndex,
+        # the following code is the equivalent implementation in the pandas (CPU) case.
+        series = cudf.Series(range(len(data)))
+        # reset_index(drop=True) is required,
+        # since series groupby will raise an exception if the index has duplicate values
+        groups = (data % size).reset_index(drop=True)
+        groupby = series.groupby(groups)
+        result = []
+        for i in range(size):
+            try:
+                idx = cudf.Index(groupby.get_group(i).drop_duplicates())
+            except KeyError:
+                idx = cudf.Index([])
+            result.append(idx)
+        return result
+    else:
+        idx_to_grouped = pd.RangeIndex(0, len(data)).groupby(data % size)
+        return [idx_to_grouped.get(i, pd.Index([])) for i in range(size)]
+
+
+def _get_index_map(
+    data: Union[pd.Series, pd.DataFrame, "cudf.DataFrame", "cudf.Series"], on: Callable
+) -> Union["cudf.Index", pd.Index]:
+    if is_cudf(data):
+        index = data.index
+        if not isinstance(index, cudf.MultiIndex):
+            # TODO: If ``on`` makes data change dimension, cudf would raise an error
+            return cudf.Index(index.to_series().map(on).values)
+        else:
+            # For cudf MultiIndex,
+            # cudf does not implement ``to_series`` method,
+            # if converted to dataframe first,
+            # df apply method in cudf has many restrictions,
+            # for example, does not support lambda function, etc.
+            # Therefore, cannot find a suitable way to implement ``MultiIndex.map`` for now.
+            raise NotImplementedError(
+                "Cannot support cudf.MultiIndex map method for now."
+            )
+    else:
+        return data.index.map(on)
+
+
 def hash_dataframe_on(df, on, size, level=None):
     if on is None:
         idx = df.index
         if level is not None:
             idx = idx.to_frame(False)[level]
-        if cudf and isinstance(idx, cudf.Index):  # pragma: no cover
-            idx = idx.to_pandas()
-        hashed_label = pd.util.hash_pandas_object(idx, categorize=False)
+        hashed_label = _get_hash(idx, categorize=False)
     elif callable(on):
         # todo optimization can be added, if ``on`` is a numpy ufunc or sth can be vectorized
-        hashed_label = pd.util.hash_pandas_object(df.index.map(on), categorize=False)
+        # TODO: cudf index may not have attribute ``map``
+        hashed_label = _get_hash(_get_index_map(df, on), categorize=False)
     else:
         if isinstance(on, list):
             to_concat = []
             for v in on:
-                if isinstance(v, pd.Series):
+                if is_series(v):
                     to_concat.append(v)
                 else:
                     to_concat.append(df[v])
-            data = pd.concat(to_concat, axis=1)
+            xpd = cudf if is_cudf(df) else pd
+            data = xpd.concat(to_concat, axis=1)
         else:
             data = df[on]
-        hashed_label = pd.util.hash_pandas_object(data, index=False, categorize=False)
-    idx_to_grouped = pd.RangeIndex(0, len(hashed_label)).groupby(hashed_label % size)
-    return [idx_to_grouped.get(i, pd.Index([])) for i in range(size)]
+        hashed_label = _get_hash(data, index=False, categorize=False)
+    return _get_index_for_on(hashed_label, size)
 
 
 def hash_dtypes(dtypes, size):
@@ -308,9 +359,17 @@ def parse_index(index_value, *args, store_data=False, key=None):
             return None
 
     def _serialize_index(index):
-        tp = getattr(IndexValue, type(index).__name__)
+        extra_properties = dict(_name=index.name)
+        if not is_pandas_2():
+            tp = getattr(IndexValue, type(index).__name__)
+        else:
+            # pandas 2.0 does not have `Int64Index`, `Float64Index`, etc.
+            name = type(index).__name__
+            if name == "Index":
+                extra_properties["_dtype"] = index.dtype
+            tp = getattr(IndexValue, type(index).__name__)
         properties = _extract_property(index, tp, store_data)
-        properties["_name"] = index.name
+        properties.update(extra_properties)
         return tp(**properties)
 
     def _serialize_range_index(index):
@@ -359,6 +418,7 @@ def parse_index(index_value, *args, store_data=False, key=None):
         )
     if hasattr(index_value, "to_pandas"):  # pragma: no cover
         # convert cudf.Index to pandas
+        # TODO: may lead to performance issue here due to data copy from cuda to memory
         index_value = index_value.to_pandas()
 
     if isinstance(index_value, _get_range_index_type()):
@@ -553,16 +613,19 @@ def _generate_value(dtype, fill_value):
     return convert(fill_value)
 
 
-def build_empty_df(dtypes, index=None):
+def build_empty_df(dtypes, index=None, gpu: Optional[bool] = False):
+    xpd = cudf if gpu is True else pd
     columns = dtypes.index
     length = len(index) if index is not None else 0
     record = [[_generate_value(dtype, 1) for dtype in dtypes]] * max(1, length)
 
     # duplicate column may exist,
     # so use RangeIndex first
-    df = pd.DataFrame(record, columns=range(len(dtypes)), index=index)
+    df = xpd.DataFrame(record, columns=range(len(dtypes)), index=index)
     for i, dtype in enumerate(dtypes):
         s = df.iloc[:, i]
+        # Even if ``s`` is a cudf object, ``s.dtype`` is always a numpy type,
+        # so ``pd.api.types.is_dtype_equal`` method is called properly here.
         if not pd.api.types.is_dtype_equal(s.dtype, dtype):
             df.iloc[:, i] = s.astype(dtype)
 
@@ -619,9 +682,10 @@ def build_df(df_obj, fill_value=1, size=1, ensure_string=False):
     return ret_df
 
 
-def build_empty_series(dtype, index=None, name=None):
+def build_empty_series(dtype, index=None, name=None, gpu: Optional[bool] = False):
+    xpd = cudf if gpu is True else pd
     length = len(index) if index is not None else 0
-    return pd.Series(
+    return xpd.Series(
         [_generate_value(dtype, 1) for _ in range(length)],
         dtype=dtype,
         index=index,
@@ -1173,8 +1237,7 @@ def fetch_corner_data(df_or_series, session=None) -> pd.DataFrame:
         head = iloc(df_or_series)[:index_size]
         tail = iloc(df_or_series)[-index_size:]
         head_data, tail_data = ExecutableTuple([head, tail]).fetch(session=session)
-        xdf = cudf if head.op.is_gpu() else pd
-        return xdf.concat([head_data, tail_data], axis="index")
+        return pd.concat([head_data, tail_data], axis="index")
 
 
 class ReprSeries(pd.Series):
@@ -1239,7 +1302,7 @@ def arrow_table_to_pandas_dataframe(arrow_table, use_arrow_dtype=True, **kw):
         # if not use arrow string, just return
         return arrow_table.to_pandas(**kw)
 
-    from .arrays import ArrowStringArray, ArrowListArray
+    from .arrays import ArrowListArray, ArrowStringArray
 
     table: pa.Table = arrow_table
     schema: pa.Schema = arrow_table.schema
@@ -1353,6 +1416,16 @@ def is_cudf(x):
     return False
 
 
+def get_storage_level_gpu_or_memory(data: Any) -> StorageLevel:
+    if isinstance(data, (tuple, list, set)):
+        return (
+            StorageLevel.GPU
+            if any([is_cudf(v) or is_cupy(v) for v in data])
+            else StorageLevel.MEMORY
+        )
+    return StorageLevel.GPU if is_cudf(data) or is_cupy(data) else StorageLevel.MEMORY
+
+
 def auto_merge_chunks(
     ctx: Context,
     df_or_series: TileableType,
@@ -1446,128 +1519,6 @@ def auto_merge_chunks(
     return new_op.new_tileable(df_or_series.op.inputs, kws=[params])
 
 
-# TODO: clean_up_func, is_on_ray and restore_func functions may be
-# removed or refactored in the future to calculate func size
-# with more accuracy as well as address some serialization issues.
-def clean_up_func(op):
-    threshold = int(os.getenv("MARS_CLOSURE_CLEAN_UP_BYTES_THRESHOLD", 10**4))
-    if threshold == -1:  # pragma: no cover
-        return
-    ctx = get_context()
-    if ctx is None:
-        return
-
-    # Note: op.func_key is set only when func was put into storage.
-    # Under ray backend, func will be put into storage.
-    # While under mars backend, since storage service is empty on supervisor,
-    # func won't be put into storage but serialized in advance to reduce upcoming
-    # expenses brought by serializations and deserializations during subtask transmission.
-    if whether_to_clean_up(op, threshold) is True:
-        assert (
-            op.logic_key is not None
-        ), f"Logic key of {op} wasn't calculated before cleaning up func."
-        logger.info("%s is cleaning up func %s.", op, op.func)
-        if is_on_ray(ctx):
-            import ray
-
-            op.func_key = ray.put(op.func)
-            logger.info("%s func %s is replaced by %s.", op, op.func, op.func_key)
-            op.func = None
-        else:
-            op.func = cloudpickle.dumps(op.func)
-
-
-def whether_to_clean_up(op, threshold):
-    func = op.func
-    counted_bytes = 0
-    max_recursion_depth = 2
-
-    from numbers import Number
-    from collections import deque
-
-    BYPASS_CLASSES = (str, bytes, Number, range, bytearray, pd.DataFrame, pd.Series)
-
-    class GetSizeEarlyStopException(Exception):
-        pass
-
-    def check_exceed_threshold():
-        nonlocal threshold, counted_bytes
-        if counted_bytes >= threshold:
-            raise GetSizeEarlyStopException()
-
-    def getsize(obj_outer):
-        _seen_obj_ids = set()
-
-        def inner_count(obj, recursion_depth):
-            obj_id = id(obj)
-            if obj_id in _seen_obj_ids or recursion_depth > max_recursion_depth:
-                return 0
-            _seen_obj_ids.add(obj_id)
-            recursion_depth += 1
-            size = sys.getsizeof(obj)
-            if isinstance(obj, BYPASS_CLASSES):
-                return size
-            elif isinstance(obj, (tuple, list, set, deque)):
-                size += sum(inner_count(i, recursion_depth) for i in obj)
-            elif hasattr(obj, "items"):
-                size += sum(
-                    inner_count(k, recursion_depth) + inner_count(v, recursion_depth)
-                    for k, v in getattr(obj, "items")()
-                )
-            if hasattr(obj, "__dict__"):
-                size += inner_count(vars(obj), recursion_depth)
-            if hasattr(obj, "__slots__"):
-                size += sum(
-                    inner_count(getattr(obj, s), recursion_depth)
-                    for s in obj.__slots__
-                    if hasattr(obj, s)
-                )
-            return size
-
-        return inner_count(obj_outer, 0)
-
-    try:
-        # Note: In most cases, func is just a function with closure, while chances are that
-        # func is a callable that doesn't have __closure__ attribute.
-        if inspect.isclass(func):
-            pass
-        elif hasattr(func, "__closure__") and func.__closure__ is not None:
-            for cell in func.__closure__:
-                counted_bytes += getsize(cell.cell_contents)
-                check_exceed_threshold()
-        elif callable(func):
-            if hasattr(func, "__dict__"):
-                for k, v in func.__dict__.items():
-                    counted_bytes += sum([getsize(k), getsize(v)])
-                    check_exceed_threshold()
-            if hasattr(func, "__slots__"):
-                for slot in func.__slots__:
-                    counted_bytes += (
-                        getsize(getattr(func, slot)) if hasattr(func, slot) else 0
-                    )
-                    check_exceed_threshold()
-    except GetSizeEarlyStopException:
-        logger.debug("Func needs cleanup.")
-        op.need_clean_up_func = True
-    else:
-        assert op.need_clean_up_func is False
-        logger.debug("Func doesn't need cleanup.")
-
-    return op.need_clean_up_func
-
-
-def restore_func(ctx: Context, op):
-    if op.need_clean_up_func and ctx is not None:
-        logger.info("%s is restoring func from %s.", op, op.func_key)
-        if is_on_ray(ctx):
-            import ray
-
-            op.func = ray.get(op.func_key)
-            logger.info("%s func %s is restored.", op, op.func)
-        else:
-            op.func = cloudpickle.loads(op.func)
-
-
 def concat_on_columns(objs: List) -> Any:
     xdf = get_xdf(objs[0])
     # In cudf, concat with axis=1 and ignore_index=False by default behaves opposite to pandas.
@@ -1577,3 +1528,7 @@ def concat_on_columns(objs: List) -> Any:
     if xdf is cudf:
         result.index = objs[0].index
     return result
+
+
+def is_pandas_2():
+    return pd.__version__ >= "2.0.0"
