@@ -1,4 +1,5 @@
-# Copyright 1999-2021 Alibaba Group Holding Ltd.
+# Copyright 2022-2023 XProbe Inc.
+# derived from copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,20 +19,20 @@ import pandas as pd
 from .... import opcodes
 from ....core import recursive_tile
 from ....serialization.serializables import (
-    FieldTypes,
     AnyField,
-    Int64Field,
     BoolField,
-    StringField,
-    Int32Field,
-    KeyField,
-    TupleField,
     DictField,
+    FieldTypes,
+    Int32Field,
+    Int64Field,
+    KeyField,
     ListField,
+    StringField,
+    TupleField,
 )
-from ....utils import lazy_import, has_unknown_shape, pd_release_version, calc_nsplits
-from ...operands import DataFrameOperand, DataFrameOperandMixin
+from ....utils import calc_nsplits, has_unknown_shape, lazy_import, pd_release_version
 from ...core import DATAFRAME_TYPE
+from ...operands import DataFrameOperand, DataFrameOperandMixin
 from ...utils import build_empty_df, build_empty_series, parse_index
 
 cudf = lazy_import("cudf")
@@ -242,7 +243,8 @@ class DataFrameRollingAgg(DataFrameOperand, DataFrameOperandMixin):
 
         # find prev chunks
         i = cur_chunk_index[axis]
-        rest = window if not center else window // 2
+        # handle all the possibilities of op.closed.
+        rest = window if not center else window // 2 + 1
         prev_chunks = []
         while i > 0 and rest > 0:
             prev_chunk_index = list(cur_chunk_index)
@@ -271,7 +273,8 @@ class DataFrameRollingAgg(DataFrameOperand, DataFrameOperandMixin):
 
         # find succ chunks
         j = cur_chunk_index[axis]
-        rest = 0 if not center else window - window // 2 - 1
+        # handle all the possibilities of op.closed.
+        rest = 0 if not center else window - window // 2
         chunk_size = inp.chunk_shape[axis]
         succ_chunks = []
         while j < chunk_size - 1 and rest > 0:
@@ -340,7 +343,7 @@ class DataFrameRollingAgg(DataFrameOperand, DataFrameOperandMixin):
         return prev_chunks, []
 
     @classmethod
-    def tile(cls, op):
+    def tile(cls, op: "DataFrameRollingAgg"):
         inp = op.input
         out = op.outputs[0]
         is_window_int = op.win_type != "freq"
@@ -348,6 +351,12 @@ class DataFrameRollingAgg(DataFrameOperand, DataFrameOperandMixin):
         input_ndim = inp.ndim
         output_ndim = out.ndim
 
+        from ..rolling.core import _PAIRWISE_AGG
+
+        is_pairwise_agg = op.func in _PAIRWISE_AGG and op.func_kwargs.get(
+            "pairwise", True
+        )
+
         # check if can be tiled
         inp = yield from cls._check_can_be_tiled(op, is_window_int)
 
@@ -401,12 +410,18 @@ class DataFrameRollingAgg(DataFrameOperand, DataFrameOperandMixin):
                     chunk_params["index_value"] = inp_chunk.index_value
                     chunk_params["columns_value"] = out.columns_value
                 else:
-                    if axis == 0:
+                    if is_pairwise_agg:
+                        # pairwise aggregation is axis irrelevant.
                         out_shape = list(out.shape)
-                        out_shape[axis] = inp_chunk.shape[axis]
+                        out_shape[0] = inp_chunk.shape[0] * out_shape[1]
                         chunk_params["shape"] = tuple(out_shape)
                     else:
-                        chunk_params["shape"] = inp_chunk.shape
+                        if axis == 0:
+                            out_shape = list(out.shape)
+                            out_shape[axis] = inp_chunk.shape[axis]
+                            chunk_params["shape"] = tuple(out_shape)
+                        else:
+                            chunk_params["shape"] = inp_chunk.shape
                     chunk_params["index_value"] = (
                         inp_chunk.index_value if axis == 0 else out.index_value
                     )
@@ -431,13 +446,19 @@ class DataFrameRollingAgg(DataFrameOperand, DataFrameOperandMixin):
         if out.ndim == 1:
             params["shape"] = (inp.shape[0],)
         else:
-            params["shape"] = (inp.shape[0], params["shape"][1])
+            if is_pairwise_agg:
+                params["shape"] = (
+                    inp.shape[0] * params["shape"][1],
+                    params["shape"][1],
+                )
+            else:
+                params["shape"] = (inp.shape[0], params["shape"][1])
         params["nsplits"] = calc_nsplits({c.index: c.shape for c in out_chunks})
         new_op = op.copy()
         return new_op.new_tileables([inp], kws=[params])
 
     @classmethod
-    def execute(cls, ctx, op):
+    def execute(cls, ctx, op: "DataFrameRollingAgg"):
         inp = ctx[op.input.key]
         axis = op.axis
         win_type = op.win_type
@@ -446,6 +467,12 @@ class DataFrameRollingAgg(DataFrameOperand, DataFrameOperandMixin):
             win_type = None
             window = pd.Timedelta(window)
 
+        from ..rolling.core import _PAIRWISE_AGG
+
+        is_pairwise_agg = op.func in _PAIRWISE_AGG and op.func_kwargs.get(
+            "pairwise", True
+        )
+
         preds = [ctx[pred.key] for pred in op.preds]
         pred_size = sum(pred.shape[axis] for pred in preds)
         succs = [ctx[succ.key] for succ in op.succs]
@@ -482,7 +509,13 @@ class DataFrameRollingAgg(DataFrameOperand, DataFrameOperandMixin):
 
         if pred_size > 0 or succ_size > 0:
             slc = [slice(None)] * result.ndim
-            slc[axis] = slice(pred_size, result.shape[axis] - succ_size)
+            if is_pairwise_agg:
+                slc[axis] = slice(
+                    pred_size * len(inp.dtypes),
+                    result.shape[axis] - succ_size * len(inp.dtypes),
+                )
+            else:
+                slc[axis] = slice(pred_size, result.shape[axis] - succ_size)
             result = result.iloc[tuple(slc)]
 
         ctx[op.outputs[0].key] = result
