@@ -1,4 +1,5 @@
-# Copyright 1999-2021 Alibaba Group Holding Ltd.
+# Copyright 2022-2023 XProbe Inc.
+# derived from copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,10 +16,8 @@
 import asyncio
 import concurrent.futures
 import itertools
-import logging
 import json
-import random
-import string
+import logging
 import threading
 import time
 import warnings
@@ -27,44 +26,41 @@ from collections import defaultdict
 from dataclasses import dataclass
 from functools import wraps
 from numbers import Integral
-from urllib.parse import urlparse
-from weakref import ref, WeakKeyDictionary, WeakSet
 from typing import Any, Callable, Coroutine, Dict, List, Optional, Tuple, Union
+from urllib.parse import urlparse
+from weakref import WeakKeyDictionary, WeakSet, ref
 
-import numpy as np
+import xoscar as mo
+from xoscar.metrics import Metrics
 
-from ... import oscar as mo
-from ...utils import Timer
 from ...config import options
-from ...core import ChunkType, TileableType, TileableGraph, enter_mode
-from ...core.entrypoints import init_extension_entrypoints
+from ...core import ChunkType, TileableGraph, TileableType, enter_mode
 from ...core.operand import Fetch
 from ...lib.aio import (
-    alru_cache,
     Isolation,
+    alru_cache,
     get_isolation,
     new_isolation,
     stop_isolation,
 )
-from ...metrics import Metrics
 from ...services.cluster import AbstractClusterAPI, ClusterAPI
 from ...services.lifecycle import AbstractLifecycleAPI, LifecycleAPI
-from ...services.meta import MetaAPI, AbstractMetaAPI
+from ...services.meta import AbstractMetaAPI, MetaAPI
 from ...services.session import AbstractSessionAPI, SessionAPI
-from ...services.mutable import MutableAPI, MutableTensor
 from ...services.storage import StorageAPI
 from ...services.task import AbstractTaskAPI, TaskAPI, TaskResult
 from ...services.task.execution.api import Fetcher
 from ...services.web import OscarWebAPI
-from ...typing import ClientType, BandType
+from ...typing import BandType, ClientType
 from ...utils import (
+    Timer,
+    build_fetch,
+    classproperty,
+    copy_tileables,
     implements,
     merge_chunks,
     merged_chunk_as_tileable_type,
     register_asyncio_task_timeout_detector,
-    classproperty,
-    copy_tileables,
-    build_fetch,
 )
 
 logger = logging.getLogger(__name__)
@@ -380,55 +376,6 @@ class AbstractAsyncSession(AbstractSession, metaclass=ABCMeta):
         name : str
         """
 
-    @abstractmethod
-    async def create_mutable_tensor(
-        self,
-        shape: tuple,
-        dtype: Union[np.dtype, str],
-        name: str = None,
-        default_value: Union[int, float] = 0,
-        chunk_size: Union[int, Tuple] = None,
-    ):
-        """
-        Create a mutable tensor.
-
-        Parameters
-        ----------
-        shape: tuple
-            Shape of the mutable tensor.
-
-        dtype: np.dtype or str
-            Data type of the mutable tensor.
-
-        name: str, optional
-            Name of the mutable tensor, a random name will be used if not specified.
-
-        default_value: optional
-            Default value of the mutable tensor. Default is 0.
-
-        chunk_size: int or tuple, optional
-            Chunk size of the mutable tensor.
-
-        Returns
-        -------
-            MutableTensor
-        """
-
-    @abstractmethod
-    async def get_mutable_tensor(self, name: str):
-        """
-        Get a mutable tensor by name.
-
-        Parameters
-        ----------
-        name: str
-            Name of the mutable tensor to get.
-
-        Returns
-        -------
-            MutableTensor
-        """
-
     async def stop_server(self):
         """
         Stop server.
@@ -599,55 +546,6 @@ class AbstractSyncSession(AbstractSession, metaclass=ABCMeta):
             web endpoint
         """
 
-    @abstractmethod
-    def create_mutable_tensor(
-        self,
-        shape: tuple,
-        dtype: Union[np.dtype, str],
-        name: str = None,
-        default_value: Union[int, float] = 0,
-        chunk_size: Union[int, Tuple] = None,
-    ):
-        """
-        Create a mutable tensor.
-
-        Parameters
-        ----------
-        shape: tuple
-            Shape of the mutable tensor.
-
-        dtype: np.dtype or str
-            Data type of the mutable tensor.
-
-        name: str, optional
-            Name of the mutable tensor, a random name will be used if not specified.
-
-        default_value: optional
-            Default value of the mutable tensor. Default is 0.
-
-        chunk_size: int or tuple, optional
-            Chunk size of the mutable tensor.
-
-        Returns
-        -------
-            MutableTensor
-        """
-
-    @abstractmethod
-    def get_mutable_tensor(self, name: str):
-        """
-        Get a mutable tensor by name.
-
-        Parameters
-        ----------
-        name: str
-            Name of the mutable tensor to get.
-
-        Returns
-        -------
-            MutableTensor
-        """
-
     def fetch_log(
         self,
         tileables: List[TileableType],
@@ -758,7 +656,6 @@ class _IsolatedSession(AbstractAsyncSession):
         meta_api: AbstractMetaAPI,
         lifecycle_api: AbstractLifecycleAPI,
         task_api: AbstractTaskAPI,
-        mutable_api: MutableAPI,
         cluster_api: AbstractClusterAPI,
         web_api: Optional[OscarWebAPI],
         client: ClientType = None,
@@ -771,7 +668,6 @@ class _IsolatedSession(AbstractAsyncSession):
         self._task_api = task_api
         self._meta_api = meta_api
         self._lifecycle_api = lifecycle_api
-        self._mutable_api = mutable_api
         self._cluster_api = cluster_api
         self._web_api = web_api
         self.client = client
@@ -802,13 +698,12 @@ class _IsolatedSession(AbstractAsyncSession):
         session_api = await SessionAPI.create(address)
         if new:
             # create new session
-            session_address = await session_api.create_session(session_id)
+            session_id, session_address = await session_api.create_session(session_id)
         else:
             session_address = await session_api.get_session_address(session_id)
         lifecycle_api = await LifecycleAPI.create(session_id, session_address)
         meta_api = await MetaAPI.create(session_id, session_address)
         task_api = await TaskAPI.create(session_id, session_address)
-        mutable_api = await MutableAPI.create(session_id, session_address)
         cluster_api = await ClusterAPI.create(session_address)
         try:
             web_api = await OscarWebAPI.create(session_address)
@@ -822,7 +717,6 @@ class _IsolatedSession(AbstractAsyncSession):
             meta_api,
             lifecycle_api,
             task_api,
-            mutable_api,
             cluster_api,
             web_api,
             timeout=timeout,
@@ -1016,8 +910,8 @@ class _IsolatedSession(AbstractAsyncSession):
     def _get_to_fetch_tileable(
         self, tileable: TileableType
     ) -> Tuple[TileableType, List[Union[slice, Integral]]]:
-        from ...tensor.indexing import TensorIndex
         from ...dataframe.indexing.iloc import DataFrameIlocGetItem, SeriesIlocGetItem
+        from ...tensor.indexing import TensorIndex
 
         slice_op_types = TensorIndex, DataFrameIlocGetItem, SeriesIlocGetItem
 
@@ -1085,11 +979,9 @@ class _IsolatedSession(AbstractAsyncSession):
         return storage_api
 
     async def fetch(self, *tileables, **kwargs) -> list:
-        if kwargs:  # pragma: no cover
-            unexpected_keys = ", ".join(list(kwargs.keys()))
-            raise TypeError(f"`fetch` got unexpected arguments: {unexpected_keys}")
-
-        fetcher = Fetcher.create(self._backend, get_storage_api=self._get_storage_api)
+        fetcher = Fetcher.create(
+            self._backend, get_storage_api=self._get_storage_api, **kwargs
+        )
 
         with enter_mode(build=True):
             chunks = []
@@ -1308,23 +1200,6 @@ class _IsolatedSession(AbstractAsyncSession):
     async def destroy_remote_object(self, session_id: str, name: str):
         return await self._session_api.destroy_remote_object(session_id, name)
 
-    async def create_mutable_tensor(
-        self,
-        shape: tuple,
-        dtype: Union[np.dtype, str],
-        name: str = None,
-        default_value: Union[int, float] = 0,
-        chunk_size: Union[int, Tuple] = None,
-    ):
-        tensor_info = await self._mutable_api.create_mutable_tensor(
-            shape, dtype, name, default_value, chunk_size
-        )
-        return tensor_info, self._mutable_api
-
-    async def get_mutable_tensor(self, name: str):
-        tensor_info = await self._mutable_api.get_mutable_tensor(name)
-        return tensor_info, self._mutable_api
-
     async def stop_server(self):
         if self.client:
             await self.client.stop()
@@ -1341,21 +1216,19 @@ class _IsolatedWebSession(_IsolatedSession):
         timeout: float = None,
         request_rewriter: Callable = None,
     ):
-        from ...services.session import WebSessionAPI
+        from ...services.cluster import WebClusterAPI
         from ...services.lifecycle import WebLifecycleAPI
         from ...services.meta import WebMetaAPI
+        from ...services.session import WebSessionAPI
         from ...services.task import WebTaskAPI
-        from ...services.mutable import WebMutableAPI
-        from ...services.cluster import WebClusterAPI
 
         session_api = WebSessionAPI(address, request_rewriter)
         if new:
             # create new session
-            await session_api.create_session(session_id)
+            session_id, _ = await session_api.create_session(session_id)
         lifecycle_api = WebLifecycleAPI(session_id, address, request_rewriter)
         meta_api = WebMetaAPI(session_id, address, request_rewriter)
         task_api = WebTaskAPI(session_id, address, request_rewriter)
-        mutable_api = WebMutableAPI(session_id, address, request_rewriter)
         cluster_api = WebClusterAPI(address, request_rewriter)
 
         return cls(
@@ -1366,7 +1239,6 @@ class _IsolatedWebSession(_IsolatedSession):
             meta_api,
             lifecycle_api,
             task_api,
-            mutable_api,
             cluster_api,
             None,
             timeout=timeout,
@@ -1444,7 +1316,9 @@ class AsyncSession(AbstractAsyncSession):
         coro = _IsolatedSession.init(address, session_id, backend, new=new, **kwargs)
         fut = asyncio.run_coroutine_threadsafe(coro, isolation.loop)
         isolated_session = await asyncio.wrap_future(fut)
-        return AsyncSession(address, session_id, isolated_session, isolation)
+        return AsyncSession(
+            address, isolated_session.session_id, isolated_session, isolation
+        )
 
     def as_default(self) -> AbstractSession:
         AbstractSession._default = self._isolated_session
@@ -1510,25 +1384,6 @@ class AsyncSession(AbstractAsyncSession):
     async def destroy_remote_object(self, session_id: str, name: str):
         pass  # pragma: no cover
 
-    @implements(AbstractAsyncSession.create_mutable_tensor)
-    async def create_mutable_tensor(
-        self,
-        shape: tuple,
-        dtype: Union[np.dtype, str],
-        name: str = None,
-        default_value: Union[int, float] = 0,
-        chunk_size: Union[int, Tuple] = None,
-    ):
-        tensor_info, mutable_api = await self._isolated_session.create_mutable_tensor(
-            shape, dtype, name, default_value, chunk_size
-        )
-        return MutableTensor.create(tensor_info, mutable_api, self._loop)
-
-    @implements(AbstractAsyncSession.get_mutable_tensor)
-    async def get_mutable_tensor(self, name: str):
-        tensor_info, mutable_api = await self._isolated_session.get_mutable_tensor(name)
-        return MutableTensor.create(tensor_info, mutable_api, self._loop)
-
     @implements(AbstractAsyncSession.get_web_endpoint)
     @_delegate_to_isolated_session
     async def get_web_endpoint(self) -> Optional[str]:
@@ -1619,7 +1474,9 @@ class SyncSession(AbstractSyncSession):
         coro = _IsolatedSession.init(address, session_id, backend, new=new, **kwargs)
         fut = asyncio.run_coroutine_threadsafe(coro, isolation.loop)
         isolated_session = fut.result()
-        return SyncSession(address, session_id, isolated_session, isolation)
+        return SyncSession(
+            address, isolated_session.session_id, isolated_session, isolation
+        )
 
     def as_default(self) -> AbstractSession:
         AbstractSession._default = self._isolated_session
@@ -1746,29 +1603,6 @@ class SyncSession(AbstractSyncSession):
     def get_cluster_versions(self) -> List[str]:
         pass  # pragma: no cover
 
-    @implements(AbstractSyncSession.create_mutable_tensor)
-    def create_mutable_tensor(
-        self,
-        shape: tuple,
-        dtype: Union[np.dtype, str],
-        name: str = None,
-        default_value: Union[int, float] = 0,
-        chunk_size: Union[int, Tuple] = None,
-    ):
-        coro = self._isolated_session.create_mutable_tensor(
-            shape, dtype, name, default_value, chunk_size
-        )
-        fut = asyncio.run_coroutine_threadsafe(coro, self._loop)
-        tensor_info, mutable_api = fut.result()
-        return MutableTensor.create(tensor_info, mutable_api, self._loop)
-
-    @implements(AbstractSyncSession.get_mutable_tensor)
-    def get_mutable_tensor(self, name: str):
-        coro = self._isolated_session.get_mutable_tensor(name)
-        fut = asyncio.run_coroutine_threadsafe(coro, self._loop)
-        tensor_info, mutable_api = fut.result()
-        return MutableTensor.create(tensor_info, mutable_api, self._loop)
-
     def destroy(self):
         coro = self._isolated_session.destroy()
         asyncio.run_coroutine_threadsafe(coro, self._loop).result()
@@ -1977,12 +1811,6 @@ def ensure_isolation_created(kwargs):
         return new_isolation(loop=loop)
 
 
-def _new_session_id():
-    return "".join(
-        random.choice(string.ascii_letters + string.digits) for _ in range(24)
-    )
-
-
 async def _new_session(
     address: str,
     session_id: str = None,
@@ -1990,9 +1818,6 @@ async def _new_session(
     default: bool = False,
     **kwargs,
 ) -> AbstractSession:
-    if session_id is None:
-        session_id = _new_session_id()
-
     session = await AsyncSession.init(
         address, session_id=session_id, backend=backend, new=True, **kwargs
     )
@@ -2009,8 +1834,6 @@ def new_session(
     new: bool = True,
     **kwargs,
 ) -> AbstractSession:
-    # load third party extensions.
-    init_extension_entrypoints()
     ensure_isolation_created(kwargs)
 
     if address is None:
@@ -2018,9 +1841,6 @@ def new_session(
         if "init_local" not in kwargs:
             kwargs["init_local"] = True
 
-    if session_id is None:
-        session_id = _new_session_id()
-
     session = SyncSession.init(
         address, session_id=session_id, backend=backend, new=new, **kwargs
     )
