@@ -1,4 +1,5 @@
-# Copyright 1999-2021 Alibaba Group Holding Ltd.
+# Copyright 2022-2023 XProbe Inc.
+# derived from copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,11 +20,11 @@ try:
         from ._k_means_common import KMeansInertia, KMeansRelocateEmptyClusters
         from ._k_means_elkan_iter import (
             KMeansElkanInitBounds,
-            KMeansElkanUpdate,
             KMeansElkanPostprocess,
+            KMeansElkanUpdate,
         )
         from ._k_means_init import KMeansPlusPlusInit
-        from ._k_means_lloyd_iter import KMeansLloydUpdate, KMeansLloydPostprocess
+        from ._k_means_lloyd_iter import KMeansLloydPostprocess, KMeansLloydUpdate
 
         del (
             KMeansInertia,
