@@ -1,4 +1,5 @@
-# Copyright 1999-2021 Alibaba Group Holding Ltd.
+# Copyright 2022-2023 XProbe Inc.
+# derived from copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,10 +18,10 @@ import numpy as np
 from ... import opcodes
 from ...core import OutputType, recursive_tile
 from ...core.operand import OperandStage
-from ...serialization.serializables import KeyField, Int32Field, BoolField
-from ...tensor.array_utils import as_same_device, device, cp, sparse
+from ...serialization.serializables import BoolField, Int32Field, KeyField
+from ...tensor.array_utils import as_same_device, cp, device, sparse
 from ...tensor.core import TensorOrder
-from ...utils import has_unknown_shape
+from ...utils import has_unknown_shape, is_same_module
 from ..operands import LearnOperand, LearnOperandMixin
 from ._k_means_common import _execute_merge_update, _relocate_empty_clusters
 from ._k_means_elkan import (
@@ -192,7 +193,7 @@ class KMeansElkanInitBounds(LearnOperand, LearnOperandMixin):
         )
 
         with device(device_id):
-            if xp is cp:  # pragma: no cover
+            if is_same_module(xp, cp):  # pragma: no cover
                 raise NotImplementedError("cannot support init_bounds for kmeans elkan")
 
             n_samples = x.shape[0]
@@ -202,10 +203,10 @@ class KMeansElkanInitBounds(LearnOperand, LearnOperandMixin):
             upper_bounds = np.zeros(n_samples, dtype=x.dtype)
             lower_bounds = np.zeros((n_samples, n_clusters), dtype=x.dtype)
 
-            if xp is np:
+            if is_same_module(xp, np):
                 init_bounds = init_bounds_dense
             else:
-                assert xp is sparse
+                assert is_same_module(xp, sparse)
                 init_bounds = init_bounds_sparse
 
             init_bounds(
@@ -402,15 +403,9 @@ class KMeansElkanUpdate(LearnOperand, LearnOperandMixin):
         out_chunks = [list() for _ in range(op.output_limit)]
         for i in range(x.chunk_shape[0]):
             x_chunk = x.cix[i, 0]
-            sample_weight_chunk = sample_weight.cix[
-                i,
-            ]
-            labels_chunk = labels.cix[
-                i,
-            ]
-            upper_bounds_chunk = upper_bounds.cix[
-                i,
-            ]
+            sample_weight_chunk = sample_weight.cix[i,]
+            labels_chunk = labels.cix[i,]
+            upper_bounds_chunk = upper_bounds.cix[i,]
             lower_bounds_chunk = lower_bounds.cix[i, 0]
             chunk_op = op.copy().reset_key()
             chunk_op.stage = OperandStage.map
@@ -534,9 +529,9 @@ class KMeansElkanUpdate(LearnOperand, LearnOperandMixin):
                     centers_new = np.zeros_like(centers_old)
                 weight_in_clusters = np.zeros(op.n_clusters, dtype=x.dtype)
 
-                if xp is np:
+                if is_same_module(xp, np):
                     method = update_chunk_dense
-                elif xp is sparse:
+                elif is_same_module(xp, sparse):
                     method = update_chunk_sparse
                 else:  # pragma: no cover
                     raise NotImplementedError("Does not support run on GPU")
