@@ -1,4 +1,5 @@
-# Copyright 1999-2021 Alibaba Group Holding Ltd.
+# Copyright 2022-2023 XProbe Inc.
+# derived from copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,17 +13,17 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from .pairwise import euclidean_distances, pairwise_distances, pairwise_distances_topk
 from ._classification import (
     accuracy_score,
+    f1_score,
+    fbeta_score,
     log_loss,
     multilabel_confusion_matrix,
     precision_recall_fscore_support,
     precision_score,
     recall_score,
-    f1_score,
-    fbeta_score,
 )
-from ._ranking import roc_curve, auc, roc_auc_score
+from ._ranking import auc, roc_auc_score, roc_curve
 from ._regresssion import r2_score
 from ._scorer import get_scorer
+from .pairwise import euclidean_distances, pairwise_distances, pairwise_distances_topk
