@@ -1,4 +1,5 @@
-# Copyright 1999-2021 Alibaba Group Holding Ltd.
+# Copyright 2022-2023 XProbe Inc.
+# derived from copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,21 +17,20 @@ from functools import partial
 
 import numpy as np
 
-from .... import opcodes
-from .... import options
+from .... import opcodes, options
 from ....core import recursive_tile
 from ....core.operand import OperandStage
 from ....serialization.serializables import (
-    KeyField,
+    AnyField,
     BoolField,
     DictField,
     Int64Field,
-    AnyField,
+    KeyField,
 )
+from ....tensor.array_utils import as_same_device, device, get_array_module
 from ....tensor.core import TensorOrder
 from ....tensor.merge import TensorConcatenate
-from ....tensor.array_utils import as_same_device, device, get_array_module
-from ....utils import has_unknown_shape, ensure_own_data
+from ....utils import ensure_own_data, has_unknown_shape, is_same_module
 from ...utils import gen_batches, get_chunk_n_rows
 from ...utils.validation import _num_samples
 from .core import PairwiseDistances
@@ -78,7 +78,7 @@ def _check_chunk_size(reduced, chunk_size):  # pragma: no cover
 def _pariwise_distance_chunked(
     X, Y, reduce_func=None, metric="euclidean", working_memory=None, xp=None, **kwds
 ):
-    if xp is np:
+    if is_same_module(xp, np):
         from sklearn.metrics import pairwise_distances
     else:  # pragma: no cover
         from cuml.metrics import pairwise_distances
