@@ -1,4 +1,5 @@
-# Copyright 1999-2021 Alibaba Group Holding Ltd.
+# Copyright 2022-2023 XProbe Inc.
+# derived from copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,12 +13,12 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from .azure import AzureBlobFileSystem
 from .base import FileSystem
-from .core import file_size, glob, get_fs, open_file, register_filesystem
+from .core import file_size, get_fs, get_scheme, glob, open_file, register_filesystem
 from .fsmap import FSMap
-from .local import LocalFileSystem
 
 # noinspection PyUnresolvedReferences
 from .hdfs import HadoopFileSystem
-from .azure import AzureBlobFileSystem
+from .local import LocalFileSystem
 from .s3 import S3FileSystem
