@@ -1,5 +1,5 @@
-# -*- coding: utf-8 -*-
-# Copyright 1999-2021 Alibaba Group Holding Ltd.
+# Copyright 2022-2023 XProbe Inc.
+# derived from copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,18 +21,18 @@ import sys
 import uuid
 from collections import namedtuple
 from ctypes import (
+    CDLL,
+    POINTER,
+    Structure,
+    byref,
     c_char,
     c_char_p,
     c_int,
     c_uint,
     c_ulonglong,
-    byref,
     create_string_buffer,
-    Structure,
-    POINTER,
-    CDLL,
 )
-from typing import List, Tuple, Optional, Union
+from typing import List, Optional, Tuple, Union
 
 from ..utils import parse_readable_size
 
@@ -386,21 +386,23 @@ def get_device_info(dev_index: int) -> _cu_device_info:
     except KeyError:
         pass
 
+    _init_nvml()
+    if _nvml_lib is None:
+        return None
     _init()
     if _init_pid is None:
         return None
 
     device = c_int()
     name_buf = create_string_buffer(100)
-    uuid_t = _CUuuid_t()
     cc_major = c_int()
     cc_minor = c_int()
     cores = c_int()
     threads_per_core = c_int()
 
+    uuid_b = get_index_and_uuid(dev_index).uuid
     _cu_check_error(_cuda_lib.cuDeviceGet(byref(device), c_int(dev_index)))
     _cu_check_error(_cuda_lib.cuDeviceGetName(name_buf, len(name_buf), device))
-    _cu_check_error(_cuda_lib.cuDeviceGetUuid(byref(uuid_t), device))
     _cu_check_error(
         _cuda_lib.cuDeviceComputeCapability(byref(cc_major), byref(cc_minor), device)
     )
@@ -426,7 +428,7 @@ def get_device_info(dev_index: int) -> _cu_device_info:
 
     info = _device_infos[dev_index] = _cu_device_info(
         index=real_dev_index,
-        uuid=uuid.UUID(bytes=uuid_t.bytes),
+        uuid=uuid_b,
         name=name_buf.value.decode(),
         multiprocessors=cores.value,
         cuda_cores=cores.value
@@ -448,11 +450,9 @@ def get_device_status(dev_index: int) -> _nvml_device_status:
 
     dev_uuid = get_device_info(dev_index).uuid
 
-    uuid_str = ("GPU-" + str(dev_uuid)).encode()
-
     if not _is_wsl:
         _nvml_check_error(
-            _nvml_lib.nvmlDeviceGetHandleByUUID(uuid_str, byref(c_device))
+            _nvml_lib.nvmlDeviceGetHandleByUUID(dev_uuid, byref(c_device))
         )
 
         _nvml_check_error(
@@ -494,12 +494,33 @@ def get_device_status(dev_index: int) -> _nvml_device_status:
         if util_node.find("gpu_util").text == "N/A":
             gpu_util = 0
         else:
-            gpu_util = int(util_node.find("gpu_util"))
+            try:
+                gpu_util_element = util_node.find("gpu_util")
+                gpu_util_text = (
+                    gpu_util_element.text if gpu_util_element is not None else None
+                )
+                if gpu_util_text:
+                    gpu_util = int(gpu_util_text.split()[0])
+                else:
+                    gpu_util = 0
+                    logger.warning("gpu_util element or text not found.")
+            except Exception as e:
+                logger.warning("Failed to get the GPU util", exc_info=True)
         if util_node.find("memory_util").text == "N/A":
             mem_util = 0
         else:
-            mem_util = int(util_node.find("memory_util"))
-
+            try:
+                mem_util_element = util_node.find("memory_util")
+                mem_util_text = (
+                    mem_util_element.text if mem_util_element is not None else None
+                )
+                if mem_util_text:
+                    mem_util = int(mem_util_text.split()[0])
+                else:
+                    mem_util = 0
+                    logger.warning("memory_util element or text not found.")
+            except Exception as e:
+                logger.warning("Failed to get the memory util", exc_info=True)
         temperature = int(gpu_node.find("temperature").find("gpu_temp").text[:-1])
 
     return _nvml_device_status(
@@ -631,7 +652,7 @@ def get_compute_running_processes(handle: _nvmlDevice_t) -> List[nvmlFriendlyObj
         return []
     elif ret == NVML_ERROR_INSUFFICIENT_SIZE:
         # typical case
-        # oversize the array incase more processes are created
+        # oversize the array in case more processes are created
         c_count.value = c_count.value * 2 + 5
         proc_array = _nvmlProcessInfo_t * c_count.value
         c_procs = proc_array()
