@@ -1,6 +1,5 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
-# Copyright 1999-2021 Alibaba Group Holding Ltd.
+# Copyright 2022-2023 XProbe Inc.
+# derived from copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,16 +15,16 @@
 
 from functools import partialmethod
 
-from ...utils import ceildiv
+from ...utils import ceildiv, is_same_module
 from .core import (
-    issparse,
-    get_array_module,
-    get_sparse_module,
     cp,
     cps,
-    np,
-    naked,
+    get_array_module,
+    get_sparse_module,
     is_cupy,
+    issparse,
+    naked,
+    np,
 )
 
 
@@ -34,7 +33,6 @@ class SparseNDArray:
     __array_priority__ = 21
 
     def __new__(cls, *args, **kwargs):
-
         shape = kwargs.get("shape", None)
         if shape is not None and len(shape) == 1:
             from .vector import SparseVector
@@ -83,7 +81,7 @@ def call_sparse(method, *args, **kwargs):
     try:
         new_data = getattr(xp, method)(*new_args, **kwargs)
     except AttributeError:
-        if xp is np:
+        if is_same_module(xp, np):
             from scipy import special
         else:
             from cupyx.scipy import special
@@ -118,7 +116,7 @@ class SparseArray(SparseNDArray):
         return self.toarray()
 
     def ascupy(self):
-        is_cp = get_array_module(self.spmatrix) is cp
+        is_cp = is_same_module(get_array_module(self.spmatrix), cp)
         if is_cp:
             return self
         mat_tuple = (
@@ -131,7 +129,7 @@ class SparseArray(SparseNDArray):
         )
 
     def asscipy(self):
-        is_cp = get_array_module(self.spmatrix) is cp
+        is_cp = is_same_module(get_array_module(self.spmatrix), cp)
         if not is_cp:
             return self
         return SparseNDArray(self.spmatrix.get(), shape=self.shape)
@@ -222,7 +220,7 @@ class SparseArray(SparseNDArray):
         self.spmatrix = xps.csr_matrix(x)
 
     def __getattr__(self, attr):
-        is_cp = get_array_module(self.spmatrix) is cp
+        is_cp = is_same_module(get_array_module(self.spmatrix), cp)
         if attr == "device" and is_cp:
             try:
                 return self.spmatrix.device
@@ -682,7 +680,7 @@ class SparseArray(SparseNDArray):
     def _scipy_unary(self, func_name):
         spmatrix = self.spmatrix
         xp = get_array_module(spmatrix)
-        if xp is np:
+        if is_same_module(xp, np):
             from scipy import special
         else:
             from cupyx.scipy import special
@@ -701,7 +699,7 @@ class SparseArray(SparseNDArray):
 
         xp = get_array_module(self.spmatrix)
 
-        if xp is np:
+        if is_same_module(xp, np):
             from scipy import special
         else:  # pragma: no cover
             from cupyx.scipy import special
