@@ -1,4 +1,5 @@
-# Copyright 1999-2021 Alibaba Group Holding Ltd.
+# Copyright 2022-2023 XProbe Inc.
+# derived from copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,13 +15,10 @@
 
 from typing import Any, Dict, List
 
-try:
-    import scipy.sparse as sps
-except ImportError:  # pragma: no cover
-    sps = None
+import scipy.sparse as sps
+from xoscar.serialization.core import Serializer, buffered, deserialize, serialize
 
 from ..lib.sparse import SparseNDArray
-from .core import Serializer, serialize, deserialize, buffered
 
 
 class SparseNDArraySerializer(Serializer):
