@@ -1,4 +1,5 @@
-# Copyright 1999-2021 Alibaba Group Holding Ltd.
+# Copyright 2022-2023 XProbe Inc.
+# derived from copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,5 +14,5 @@
 # limitations under the License.
 
 from .core import AbstractMetaAPI
-from .oscar import MetaAPI, MockMetaAPI, WorkerMetaAPI, MockWorkerMetaAPI
+from .oscar import MetaAPI, MockMetaAPI, MockWorkerMetaAPI, WorkerMetaAPI
 from .web import WebMetaAPI
