@@ -1,4 +1,5 @@
-# Copyright 1999-2021 Alibaba Group Holding Ltd.
+# Copyright 2022-2023 XProbe Inc.
+# derived from copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,11 +17,12 @@ import asyncio
 import logging
 from collections import defaultdict
 from dataclasses import dataclass
-from typing import Any, Dict, List, Optional, Union, Tuple
+from typing import Any, Dict, List, Optional, Tuple, Union
+
+import xoscar as mo
+from xoscar.backends.allocate_strategy import IdleLabel, NoIdleSlot
 
-from ... import oscar as mo
 from ...lib.aio import AioFileObject
-from ...oscar.backends.allocate_strategy import IdleLabel, NoIdleSlot
 from ...resource import cuda_card_stats
 from ...storage import StorageLevel, get_storage_backend
 from ...storage.base import ObjectInfo, StorageBackend
@@ -501,7 +503,7 @@ class StorageManagerActor(mo.StatelessActor):
 
     async def _create_storage_handler_actors(self):
         from .handler import StorageHandlerActor
-        from .transfer import SenderManagerActor, ReceiverManagerActor
+        from .transfer import ReceiverManagerActor, SenderManagerActor
 
         for band_name in self._init_params:
             strategy = IdleLabel(band_name, "StorageHandler")
@@ -547,7 +549,7 @@ class StorageManagerActor(mo.StatelessActor):
 
     async def _create_transfer_actors(self):
         from .handler import StorageHandlerActor
-        from .transfer import SenderManagerActor, ReceiverManagerActor
+        from .transfer import ReceiverManagerActor, SenderManagerActor
 
         default_band_name = "numa-0"
         sender_strategy = IdleLabel("io", "sender")
@@ -620,22 +622,33 @@ class StorageManagerActor(mo.StatelessActor):
 
         if self._cluster_api is not None:
             while True:
-                upload_tasks = []
-                for band, level_to_quota in self._quotas.items():
-                    for level, quota_ref in level_to_quota.items():
-                        total, used = await quota_ref.get_quota()
-                        used = int(used)
-                        if total is not None:
-                            total = int(total)
-                        storage_info = StorageInfo(
-                            storage_level=level, total_size=total, used_size=used
-                        )
-                        upload_tasks.append(
-                            self._cluster_api.set_band_storage_info.delay(
-                                band, storage_info
+                try:
+                    upload_tasks = []
+                    for band, level_to_quota in self._quotas.items():
+                        for level, quota_ref in level_to_quota.items():
+                            total, used = await quota_ref.get_quota()
+                            used = int(used)
+                            if total is not None:
+                                total = int(total)
+                            storage_info = StorageInfo(
+                                storage_level=level, total_size=total, used_size=used
                             )
-                        )
-                await self._cluster_api.set_band_storage_info.batch(*upload_tasks)
+                            upload_tasks.append(
+                                self._cluster_api.set_band_storage_info.delay(
+                                    band, storage_info
+                                )
+                            )
+                    await self._cluster_api.set_band_storage_info.batch(*upload_tasks)
+                except asyncio.CancelledError:  # pragma: no cover
+                    break
+                except RuntimeError as ex:  # pragma: no cover
+                    if (
+                        "cannot schedule new futures after interpreter shutdown"
+                        not in str(ex)
+                    ):
+                        # when atexit is triggered, the default pool might be shutdown
+                        # and to_thread will fail
+                        break
                 await asyncio.sleep(0.5)
 
     async def upload_disk_info(self):
