@@ -1,4 +1,5 @@
-# Copyright 1999-2021 Alibaba Group Holding Ltd.
+# Copyright 2022-2023 XProbe Inc.
+# derived from copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,10 +16,11 @@
 import asyncio
 import logging
 from collections import defaultdict
-from typing import Any, Dict, List, Union
+from typing import Any, Dict, List, Optional, Union
+
+import xoscar as mo
+from xoscar.serialization import AioDeserializer
 
-from ... import oscar as mo
-from ...serialization import AioDeserializer
 from ...storage import StorageLevel, get_storage_backend
 from ...storage.core import StorageFileObject
 from ...typing import BandType
@@ -26,11 +28,11 @@ from ...utils import calc_data_size, lazy_import
 from ..cluster import ClusterAPI, StorageInfo
 from ..meta import MetaAPI
 from .core import (
-    StorageQuotaActor,
-    DataManagerActor,
     DataInfo,
-    build_data_info,
+    DataManagerActor,
+    StorageQuotaActor,
     WrappedStorageFileObject,
+    build_data_info,
 )
 from .errors import DataNotExist, NoDataToSpill
 
@@ -109,8 +111,14 @@ class StorageHandlerActor(mo.Actor):
                 )
             except NotImplementedError:
                 data = yield self._clients[data_info.level].get(data_info.object_id)
+
                 try:
-                    sliced_value = data.iloc[tuple(conditions)]
+                    from ...dataframe.utils import is_cudf
+
+                    if is_cudf(data) and len(conditions) == 1:
+                        sliced_value = data.iloc[conditions[0]]
+                    else:
+                        sliced_value = data.iloc[tuple(conditions)]
                 except AttributeError:
                     sliced_value = data[tuple(conditions)]
                 res = sliced_value
@@ -122,6 +130,7 @@ class StorageHandlerActor(mo.Actor):
         session_id: str,
         data_key: str,
         conditions: List = None,
+        to_cpu: Optional[bool] = None,
         error: str = "raise",
     ):
         try:
@@ -129,6 +138,12 @@ class StorageHandlerActor(mo.Actor):
                 session_id, data_key, self._band_name
             )
             data = yield self._get_data(data_info, conditions)
+
+            from ...dataframe.utils import is_cudf
+            from ...tensor.array_utils import is_cupy
+
+            if (is_cudf(data) or is_cupy(data)) and to_cpu is True:
+                data = data.to_pandas() if is_cudf(data) else data.get()
             raise mo.Return(data)
         except DataNotExist:
             if error == "raise":
@@ -139,21 +154,24 @@ class StorageHandlerActor(mo.Actor):
         session_id: str,
         data_key: str,
         conditions: List = None,
+        to_cpu: Optional[bool] = None,
         error: str = "raise",
     ):
         info = self._data_manager_ref.get_data_info.delay(
             session_id, data_key, self._band_name, error
         )
-        return info, conditions
+        return info, conditions, to_cpu
 
     @get.batch
     async def batch_get(self, args_list, kwargs_list):
         infos = []
         conditions_list = []
+        to_cpus = []
         for args, kwargs in zip(args_list, kwargs_list):
-            info, conditions = self._get_data_info(*args, **kwargs)
+            info, conditions, to_cpu = self._get_data_info(*args, **kwargs)
             infos.append(info)
             conditions_list.append(conditions)
+            to_cpus.append(to_cpu)
         data_infos = await self._data_manager_ref.get_data_info.batch(*infos)
         results = []
         writer_args = [
@@ -166,32 +184,31 @@ class StorageHandlerActor(mo.Actor):
             object_id_to_reader[object_id] = await self._clients[level].open_reader(
                 object_id
             )
-        for data_info, conditions in zip(data_infos, conditions_list):
-            if data_info is None:
-                results.append(None)
+        for data_info, conditions, to_cpu in zip(data_infos, conditions_list, to_cpus):
+            if data_info is None:  # pragma: no cover
+                result = None
             elif data_info.offset is not None:
                 reader = object_id_to_reader[data_info.object_id]
                 await reader.seek(data_info.offset)
                 result = await AioDeserializer(reader).run()
-                results.append(result)
             else:
                 result = yield self._get_data(data_info, conditions)
-                results.append(result)
+
+            from ...dataframe.utils import is_cudf
+            from ...tensor.array_utils import is_cupy
+
+            if (is_cudf(result) or is_cupy(result)) and to_cpu is True:
+                result = result.to_pandas() if is_cudf(result) else result.get()
+            results.append(result)
         raise mo.Return(results)
 
     def _get_default_level(self, obj):
-        obj = obj[0] if isinstance(obj, (list, tuple)) else obj
+        from ...dataframe.utils import get_storage_level_gpu_or_memory
+
         if self.highest_level != StorageLevel.GPU:
             return self.highest_level
-        else:  # pragma: no cover
-            if cudf is not None and isinstance(
-                obj, (cudf.DataFrame, cudf.Series, cudf.Index)
-            ):
-                return StorageLevel.GPU
-            elif cupy is not None and isinstance(obj, cupy.ndarray):
-                return StorageLevel.GPU
-            else:
-                return StorageLevel.MEMORY
+        else:
+            return get_storage_level_gpu_or_memory(obj)
 
     @mo.extensible
     async def put(
