@@ -1,4 +1,5 @@
-# Copyright 1999-2021 Alibaba Group Holding Ltd.
+# Copyright 2022-2023 XProbe Inc.
+# derived from copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,29 +18,29 @@ import logging
 import sys
 import time
 from collections import defaultdict
-from typing import Any, Dict, List, Optional, Set, Type, Tuple
+from typing import Any, Dict, List, Optional, Set, Tuple, Type
 
-from .... import oscar as mo
-from ....core import ChunkGraph, OperandType, enter_mode, ExecutionError
+import xoscar as mo
+from xoscar.metrics import Metrics
+from xoscar.serialization import AioSerializer
+
+from ....core import ChunkGraph, ExecutionError, OperandType, enter_mode
 from ....core.context import get_context
-from ....core.operand import (
-    Fetch,
-    FetchShuffle,
-    execute,
-)
+from ....core.operand import Fetch, FetchShuffle, execute
+from ....dataframe.utils import get_storage_level_gpu_or_memory
 from ....lib.aio import alru_cache
-from ....metrics import Metrics
 from ....optimization.physical import optimize
-from ....serialization import AioSerializer
+from ....storage import StorageLevel
 from ....typing import BandType, ChunkType
-from ....utils import get_chunk_key_to_data_keys, calc_data_size
+from ....utils import Timer, calc_data_size, get_chunk_key_to_data_keys
 from ...context import ThreadedServiceContext
 from ...meta.api import MetaAPI, WorkerMetaAPI
 from ...session import SessionAPI
 from ...storage import StorageAPI
 from ...task import TaskAPI, task_options
-from ..core import Subtask, SubtaskStatus, SubtaskResult
-from ..utils import iter_input_data_keys, iter_output_data, get_mapper_data_keys
+from ...task.task_info_collector import TaskInfoCollector
+from ..core import Subtask, SubtaskResult, SubtaskStatus
+from ..utils import get_mapper_data_keys, iter_input_data_keys, iter_output_data
 
 logger = logging.getLogger(__name__)
 
@@ -77,6 +78,7 @@ class SubtaskProcessor:
         meta_api: MetaAPI,
         worker_meta_api: WorkerMetaAPI,
         band: BandType,
+        slot_id: int,
         supervisor_address: str,
         engines: List[str] = None,
     ):
@@ -91,6 +93,7 @@ class SubtaskProcessor:
             ]
         )
         self._band = band
+        self._slot_id = slot_id
         self._supervisor_address = supervisor_address
         self._engines = engines if engines is not None else task_options.runtime_engines
 
@@ -120,6 +123,12 @@ class SubtaskProcessor:
         self._storage_api = storage_api
         self._meta_api = meta_api
         self._worker_meta_api = worker_meta_api
+        collect_task_info = subtask.extra_config and subtask.extra_config.get(
+            "collect_task_info", None
+        )
+        self._task_info_collector = TaskInfoCollector(
+            subtask.session_id, subtask.task_id, self._band[0], collect_task_info
+        )
 
         # add metrics
         self._subtask_execution_time = Metrics.gauge(
@@ -234,35 +243,41 @@ class SubtaskProcessor:
                             to_wait.set_result(fut.result())
 
                 future.add_done_callback(cb)
-
-                try:
-                    await to_wait
-                    logger.debug(
-                        "Finish executing operand: %s, chunk: %s, subtask id: %s",
-                        chunk.op,
-                        chunk,
-                        self.subtask.subtask_id,
-                    )
-                except asyncio.CancelledError:
-                    logger.debug(
-                        "Receive cancel instruction for operand: %s,"
-                        "chunk: %s, subtask id: %s",
-                        chunk.op,
-                        chunk,
-                        self.subtask.subtask_id,
-                    )
-                    # wait for this computation to finish
-                    await future
-                    # if cancelled, stop next computation
-                    logger.debug(
-                        "Cancelled operand: %s, chunk: %s, subtask id: %s",
-                        chunk.op,
-                        chunk,
-                        self.subtask.subtask_id,
-                    )
-                    self.result.status = SubtaskStatus.cancelled
-                    raise
-
+                with Timer() as timer:
+                    try:
+                        await to_wait
+                        logger.debug(
+                            "Finish executing operand: %s, chunk: %s, subtask id: %s",
+                            chunk.op,
+                            chunk,
+                            self.subtask.subtask_id,
+                        )
+                    except asyncio.CancelledError:  # pragma: no cover
+                        logger.debug(
+                            "Receive cancel instruction for operand: %s,"
+                            "chunk: %s, subtask id: %s",
+                            chunk.op,
+                            chunk,
+                            self.subtask.subtask_id,
+                        )
+                        # wait for this computation to finish
+                        await future
+                        # if cancelled, stop next computation
+                        logger.debug(
+                            "Cancelled operand: %s, chunk: %s, subtask id: %s",
+                            chunk.op,
+                            chunk,
+                            self.subtask.subtask_id,
+                        )
+                        self.result.status = SubtaskStatus.cancelled
+                        raise
+                await self._task_info_collector.append_runtime_operand_info(
+                    self.subtask,
+                    timer.start,
+                    timer.start + timer.duration,
+                    chunk,
+                    self._processor_context,
+                )
             self.set_op_progress(chunk.op.key, 1.0)
 
             for inp in chunk_graph.iter_predecessors(chunk):
@@ -295,54 +310,67 @@ class SubtaskProcessor:
 
     async def _store_data(self, chunk_graph: ChunkGraph):
         # store data into storage
-        data_key_to_puts = {}
+        # The data stored in the storage needs to be separated based on levels
+        # (here we only have two levels: memory and GPU).
+        # Otherwise, it is hard to uniformly plan the storage region when writing data in batch.
+        level_to_data_key_to_puts: Dict[StorageLevel, Dict[str, Any]] = defaultdict(
+            dict
+        )
         shuffle_key_to_data = {}
         is_storage_seekable = await self._storage_api.is_seekable()
         for key, data, _ in iter_output_data(chunk_graph, self._processor_context):
             if isinstance(key, tuple) and is_storage_seekable:
                 shuffle_key_to_data[key] = data
             else:
+                storage_level = get_storage_level_gpu_or_memory(data)
                 put = self._storage_api.put.delay(key, data)
-                data_key_to_puts[key] = put
+                level_to_data_key_to_puts[storage_level][key] = put
 
-        stored_keys = list(data_key_to_puts.keys())
-        puts = data_key_to_puts.values()
-        logger.debug(
-            "Start putting data keys: %s, subtask id: %s",
-            stored_keys,
-            self.subtask.subtask_id,
-        )
+        stored_keys = []
         data_key_to_store_size = dict()
         data_key_to_memory_size = dict()
         data_key_to_object_id = dict()
-        if puts:
-            put_infos = asyncio.create_task(self._storage_api.put.batch(*puts))
-            try:
-                store_infos = await put_infos
-                for store_key, store_info in zip(stored_keys, store_infos):
-                    data_key_to_store_size[store_key] = store_info.store_size
-                    data_key_to_memory_size[store_key] = store_info.memory_size
-                    data_key_to_object_id[store_key] = store_info.object_id
-                logger.debug(
-                    "Finish putting data keys: %s, subtask id: %s",
-                    stored_keys,
-                    self.subtask.subtask_id,
-                )
-            except asyncio.CancelledError:
-                logger.debug(
-                    "Cancelling put data keys: %s, subtask id: %s",
-                    stored_keys,
-                    self.subtask.subtask_id,
-                )
-                put_infos.cancel()
+        data_info_fmt = "data keys: %s, subtask id: %s, storage level: %s"
+        for storage_level, data_key_to_puts in level_to_data_key_to_puts.items():
+            stored_keys.extend(list(data_key_to_puts.keys()))
+            puts = data_key_to_puts.values()
+            logger.debug(
+                f"Start putting {data_info_fmt}",
+                stored_keys,
+                self.subtask.subtask_id,
+                storage_level,
+            )
+            if puts:
+                put_infos = asyncio.create_task(self._storage_api.put.batch(*puts))
+                try:
+                    store_infos = await put_infos
+                    for store_key, store_info in zip(stored_keys, store_infos):
+                        data_key_to_store_size[store_key] = store_info.store_size
+                        data_key_to_memory_size[store_key] = store_info.memory_size
+                        data_key_to_object_id[store_key] = store_info.object_id
+                    logger.debug(
+                        f"Finish putting {data_info_fmt}",
+                        stored_keys,
+                        self.subtask.subtask_id,
+                        storage_level,
+                    )
+                except asyncio.CancelledError:  # pragma: no cover
+                    logger.debug(
+                        f"Cancelling put {data_info_fmt}",
+                        stored_keys,
+                        self.subtask.subtask_id,
+                        storage_level,
+                    )
+                    put_infos.cancel()
 
-                logger.debug(
-                    "Cancelled put data keys: %s, subtask id: %s",
-                    stored_keys,
-                    self.subtask.subtask_id,
-                )
-                self.result.status = SubtaskStatus.cancelled
-                raise
+                    logger.debug(
+                        f"Cancelled put {data_info_fmt}",
+                        stored_keys,
+                        self.subtask.subtask_id,
+                        storage_level,
+                    )
+                    self.result.status = SubtaskStatus.cancelled
+                    raise
 
         if shuffle_key_to_data:
             await self._store_mapper_data(
@@ -418,7 +446,7 @@ class SubtaskProcessor:
                 self._write_aggregated_mapper_data(key_and_band, objects, shuffle_keys)
             )
         infos = await asyncio.gather(*write_tasks)
-        for (key, memory_size, store_size, object_id) in infos:
+        for key, memory_size, store_size, object_id in infos:
             data_key_to_memory_size[key] = memory_size
             data_key_to_store_size[key] = store_size
             data_key_to_object_id[key] = object_id
@@ -560,35 +588,64 @@ class SubtaskProcessor:
                 result_chunk_to_optimized[c] for c in raw_update_meta_chunks
             }
 
+            cost_times = defaultdict(tuple)
             # load inputs data
-            input_keys = await self._load_input_data()
+            with Timer() as timer:
+                input_keys = await self._load_input_data()
+            cost_times["load_data_time"] = (timer.start, timer.start + timer.duration)
+
             try:
                 # execute chunk graph
-                await self._execute_graph(chunk_graph)
+                with Timer() as timer:
+                    await self._execute_graph(chunk_graph)
+                cost_times["execute_time"] = (timer.start, timer.start + timer.duration)
             finally:
                 # unpin inputs data
                 unpinned = True
-                await self._unpin_data(input_keys)
+                with Timer() as timer:
+                    await self._unpin_data(input_keys)
+                cost_times["unpin_time"] = (timer.start, timer.start + timer.duration)
+
             # store results data
-            (
+            with Timer() as timer:
+                (
+                    stored_keys,
+                    store_sizes,
+                    memory_sizes,
+                    data_key_to_object_id,
+                ) = await self._store_data(chunk_graph)
+            cost_times["store_result_time"] = (
+                timer.start,
+                timer.start + timer.duration,
+            )
+
+            # store meta
+            with Timer() as timer:
+                await self._store_meta(
+                    chunk_graph,
+                    store_sizes,
+                    memory_sizes,
+                    data_key_to_object_id,
+                    update_meta_chunks,
+                )
+            cost_times["store_meta_time"] = (timer.start, timer.start + timer.duration)
+
+            await self._task_info_collector.append_runtime_subtask_info(
+                self.subtask,
+                self._band,
+                self._slot_id,
                 stored_keys,
                 store_sizes,
                 memory_sizes,
-                data_key_to_object_id,
-            ) = await self._store_data(chunk_graph)
-            # store meta
-            await self._store_meta(
-                chunk_graph,
-                store_sizes,
-                memory_sizes,
-                data_key_to_object_id,
-                update_meta_chunks,
+                cost_times,
             )
         except asyncio.CancelledError:
             self.result.status = SubtaskStatus.cancelled
             self.result.progress = 1.0
             raise
-        except BaseException as ex:  # noqa: E722  # nosec  # pylint: disable=bare-except
+        except (
+            BaseException
+        ) as ex:  # noqa: E722  # nosec  # pylint: disable=bare-except
             self.result.status = SubtaskStatus.errored
             self.result.progress = 1.0
             if isinstance(ex, ExecutionError):
@@ -601,6 +658,7 @@ class SubtaskProcessor:
         finally:
             if input_keys is not None and not unpinned:
                 await self._unpin_data(input_keys)
+            await self._task_info_collector.flush_subtask_info(self.subtask.subtask_id)
 
         await self.done()
         if self.result.status == SubtaskStatus.succeeded:
@@ -706,7 +764,7 @@ class SubtaskProcessorActor(mo.Actor):
         await context.init()
         return context
 
-    async def run(self, subtask: Subtask):
+    async def run(self, subtask: Subtask, slot_id: int):
         logger.info(
             "Start to run subtask: %r on %s. chunk graph contains %s",
             subtask,
@@ -726,6 +784,7 @@ class SubtaskProcessorActor(mo.Actor):
                 self._meta_api,
                 self._worker_meta_api,
                 self._band,
+                slot_id,
                 self._supervisor_address,
             )
             self._processor = self._last_processor = processor
