@@ -1,4 +1,5 @@
-# Copyright 1999-2021 Alibaba Group Holding Ltd.
+# Copyright 2022-2023 XProbe Inc.
+# derived from copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,8 +13,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from ...config import Config, is_bool, is_list, is_integer
-
+from ...config import Config, is_bool, is_integer, is_list
 
 task_options = Config()
 
@@ -24,4 +24,6 @@ task_options.register_option("fuse_enabled", True, validator=is_bool)
 task_options.register_option("reserved_finish_tasks", 25, validator=is_integer)
 
 # worker
-task_options.register_option("runtime_engines", ["numexpr", "cupy"], validator=is_list)
+task_options.register_option(
+    "runtime_engines", ["jax", "numexpr", "cupy"], validator=is_list
+)
