@@ -1,4 +1,5 @@
-# Copyright 1999-2021 Alibaba Group Holding Ltd.
+# Copyright 2022-2023 XProbe Inc.
+# derived from copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,21 +18,21 @@ import logging
 import os
 import tempfile
 import time
-from typing import Dict, Iterator, Optional, List, Set
+from typing import Dict, Iterator, List, Optional, Set
 
-from ....core import ChunkGraph, TileableGraph, Chunk, TileContext
+from xoscar.metrics import Metrics
+from xoscar.profiling import XOSCAR_ENABLE_PROFILING
+
+from ....core import Chunk, ChunkGraph, TileableGraph, TileContext
 from ....core.operand import Fetch
-from ....metrics import Metrics
 from ....optimization.logical import OptimizationRecords
-from ....oscar.profiling import (
-    ProfilingData,
-    MARS_ENABLE_PROFILING,
-)
-from ....typing import TileableType, ChunkType
+from ....profiling import ProfilingData
+from ....typing import ChunkType, TileableType
 from ....utils import Timer
-from ...subtask import SubtaskResult, Subtask
-from ..core import Task, TaskResult, TaskStatus, new_task_id, MapReduceInfo
-from ..execution.api import TaskExecutor, ExecutionChunkResult
+from ...subtask import Subtask, SubtaskResult
+from ..core import MapReduceInfo, Task, TaskResult, TaskStatus, new_task_id
+from ..execution.api import ExecutionChunkResult, TaskExecutor
+from ..task_info_collector import TaskInfoCollector
 from .preprocessor import TaskPreprocessor
 
 logger = logging.getLogger(__name__)
@@ -50,6 +51,7 @@ class TaskProcessor:
         task: Task,
         preprocessor: TaskPreprocessor,
         executor: TaskExecutor,
+        address: Optional[str],
     ):
         self._task = task
         self._preprocessor = preprocessor
@@ -59,7 +61,7 @@ class TaskProcessor:
         self._chunk_to_subtasks = dict()
         self._stage_tileables = set()
 
-        if MARS_ENABLE_PROFILING:
+        if XOSCAR_ENABLE_PROFILING:
             ProfilingData.init(task.task_id)
         elif task.extra_config and task.extra_config.get("enable_profiling"):
             ProfilingData.init(task.task_id, task.extra_config["enable_profiling"])
@@ -70,6 +72,12 @@ class TaskProcessor:
             task.extra_config and task.extra_config.get("dump_subtask_graph")
         ):
             self._dump_subtask_graph = True
+        self._collect_task_info = task.extra_config and task.extra_config.get(
+            "collect_task_info", None
+        )
+        self._task_info_collector = TaskInfoCollector(
+            self._task.session_id, self._task.task_id, address, self._collect_task_info
+        )
 
         self.result = TaskResult(
             task_id=task.task_id,
@@ -121,7 +129,11 @@ class TaskProcessor:
         return self._preprocessor.get_tiled(tileable)
 
     def get_subtasks(self, chunks: List[ChunkType]) -> List[Subtask]:
-        return [self._chunk_to_subtasks[chunk] for chunk in chunks]
+        subtasks = []
+        for chunk in chunks:
+            if chunk in self._chunk_to_subtasks:
+                subtasks.append(self._chunk_to_subtasks[chunk])
+        return subtasks
 
     def get_tileable_to_subtasks(self) -> Dict[TileableType, List[Subtask]]:
         tile_context = self.tile_context
@@ -222,9 +234,12 @@ class TaskProcessor:
                 op_to_bands=fetch_op_to_bands,
                 shuffle_fetch_type=shuffle_fetch_type,
             )
-            if self._dump_subtask_graph:
-                self._subtask_graphs.append(subtask_graph)
+            self._subtask_graphs.append(subtask_graph)
         stage_profiler.set(f"gen_subtask_graph({len(subtask_graph)})", timer.duration)
+        await self._task_info_collector.collect_subtask_operand_structure(
+            self._task, subtask_graph, stage_id
+        )
+
         logger.info(
             "Time consuming to gen a subtask graph is %ss with session id %s, task id %s, stage id %s",
             timer.duration,
@@ -371,6 +386,12 @@ class TaskProcessor:
             async with self._executor:
                 async for stage_args in self._iter_stage_chunk_graph():
                     await self._process_stage_chunk_graph(*stage_args)
+            await self._task_info_collector.collect_result_nodes(
+                self._task, self._subtask_graphs
+            )
+            await self._task_info_collector.collect_tileable_structure(
+                self._task, self.get_tileable_to_subtasks()
+            )
         except Exception as ex:
             self.result.error = ex
             self.result.traceback = ex.__traceback__
@@ -453,7 +474,7 @@ class TaskProcessor:
         self.done.set()
         if self._dump_subtask_graph:
             self.dump_subtask_graph()
-        if MARS_ENABLE_PROFILING or (
+        if XOSCAR_ENABLE_PROFILING or (
             self._task.extra_config and self._task.extra_config.get("enable_profiling")
         ):
             ProfilingData[self._task.task_id, "general"].set(
