@@ -1,4 +1,5 @@
-# Copyright 1999-2021 Alibaba Group Holding Ltd.
+# Copyright 2022-2023 XProbe Inc.
+# derived from copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,8 +15,8 @@
 
 from typing import Type
 
-from .... import oscar as mo
-from ....lib.aio import alru_cache
+import xoscar as mo
+from xoscar.aio import alru_cache
 
 
 class OscarWebAPI:
