@@ -1,4 +1,5 @@
-# Copyright 1999-2021 Alibaba Group Holding Ltd.
+# Copyright 2022-2023 XProbe Inc.
+# derived from copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,11 +24,11 @@ from ...core.operand import OperandStage
 from ...lib import sparse
 from ...lib.sparse.core import get_array_module as get_sparse_array_module
 from ...serialization.serializables import BoolField, Int32Field, Int64Field
-from ...utils import has_unknown_shape
-from ..operands import TensorMapReduceOperand, TensorOperandMixin, TensorShuffleProxy
+from ...utils import has_unknown_shape, is_same_module
 from ..array_utils import as_same_device, device
 from ..core import TensorOrder
-from ..utils import validate_axis, hash_on_axis
+from ..operands import TensorMapReduceOperand, TensorOperandMixin, TensorShuffleProxy
+from ..utils import hash_on_axis, validate_axis
 
 
 class TensorUnique(TensorMapReduceOperand, TensorOperandMixin):
@@ -321,7 +322,7 @@ class TensorUnique(TensorMapReduceOperand, TensorOperandMixin):
             inverse_ar = next(results_iter) if op.return_inverse else None
             counts_ar = next(results_iter) if op.return_counts else None
 
-            if xp is sparse:
+            if is_same_module(xp, sparse):
                 dense_xp = get_sparse_array_module(unique_ar)
             else:
                 dense_xp = xp
