@@ -1,6 +1,5 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
-# Copyright 1999-2021 Alibaba Group Holding Ltd.
+# Copyright 2022-2023 XProbe Inc.
+# derived from copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -22,12 +21,13 @@ import numpy as np
 from ... import opcodes as OperandDef
 from ...core import recursive_tile
 from ...serialization.serializables import AnyField, StringField
-from ..array_utils import as_same_device, device
+from ...utils import is_same_module
 from ..arithmetic.utils import chunk_tree_add
+from ..array_utils import as_same_device, device
 from ..core import TensorOrder
 from ..operands import TensorOperand, TensorOperandMixin
 from ..utils import decide_unify_split
-from .einsumfunc import parse_einsum_input, einsum_path
+from .einsumfunc import einsum_path, parse_einsum_input
 
 
 class TensorEinsum(TensorOperand, TensorOperandMixin):
@@ -86,11 +86,11 @@ class TensorEinsum(TensorOperand, TensorOperandMixin):
 
         # rechunk to unify nsplits
         input_nsplits = defaultdict(list)
-        for (t, axes) in tensor_axes:
+        for t, axes in tensor_axes:
             for splits, ax in zip(t.nsplits, axes):
                 input_nsplits[ax].append(splits)
         input_tensors = []
-        for (t, axes) in tensor_axes:
+        for t, axes in tensor_axes:
             new_nsplits = tuple(
                 decide_unify_split(*input_nsplits[ax])
                 if t.shape[j] > 1
@@ -120,7 +120,7 @@ class TensorEinsum(TensorOperand, TensorOperandMixin):
             tensor_shape = []
             for i, idx in enumerate(out_idx):
                 tensor_shape.append(axes_splits[output_scripts[i]][idx])
-                for (t_idx, axis) in output_axes[output_scripts[i]]:
+                for t_idx, axis in output_axes[output_scripts[i]]:
                     if input_tensors[t_idx].shape[axis] == 1:
                         all_indexes[t_idx][axis] = 0
                     else:
@@ -143,7 +143,7 @@ class TensorEinsum(TensorOperand, TensorOperandMixin):
                         ]
                 einsum_op = op.copy().reset_key()
                 in_chunks = [
-                    t.cix[tuple(indices)]
+                    t.cix[tuple([x if x is not None else indices[0] for x in indices])]
                     for t, indices in zip(input_tensors, all_indexes)
                 ]
                 chunk = einsum_op.new_chunk(
@@ -176,7 +176,7 @@ class TensorEinsum(TensorOperand, TensorOperandMixin):
         )
 
         with device(device_id):
-            if xp is np:
+            if is_same_module(xp, np):
                 ctx[op.outputs[0].key] = xp.einsum(
                     op.subscripts,
                     *inputs,
