@@ -1,6 +1,5 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
-# Copyright 1999-2021 Alibaba Group Holding Ltd.
+# Copyright 2022-2023 XProbe Inc.
+# derived from copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -27,12 +26,12 @@ import numpy as np
 
 from ...config import options
 from ...core.operand import OperandStage
-from ...serialization.serializables import KeyField, AnyField, BoolField, Int32Field
+from ...serialization.serializables import AnyField, BoolField, Int32Field, KeyField
+from ..array_utils import as_same_device, cp, device, get_array_module
 from ..core import Tensor, TensorOrder
-from ..array_utils import get_array_module, as_same_device, device, cp
-from ..utils import check_out_param, validate_axis
-from ..operands import TensorHasInput, TensorOperandMixin
 from ..datasource import tensor as astensor
+from ..operands import TensorHasInput, TensorOperandMixin
+from ..utils import check_out_param, validate_axis
 
 
 def numel(x, **kwargs):
@@ -332,7 +331,7 @@ class TensorReductionMixin(TensorOperandMixin):
             if input_chunk.size == 0 and op.keepdims:
                 # input chunk is empty, when keepdims is True, return itself
                 ret = input_chunk
-            elif "dtype" in inspect.getfullargspec(reduce_func).args:
+            elif "dtype" in inspect.signature(reduce_func).parameters:
                 ret = reduce_func(
                     input_chunk, axis=axis, dtype=op.dtype, keepdims=bool(op.keepdims)
                 )
