@@ -1,6 +1,5 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
-# Copyright 1999-2021 Alibaba Group Holding Ltd.
+# Copyright 2022-2023 XProbe Inc.
+# derived from copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -27,11 +26,10 @@ import logging
 import numbers
 import operator
 import os
-import shutil
-import weakref
 import pkgutil
 import random
 import socket
+import string
 import struct
 import sys
 import threading
@@ -39,46 +37,48 @@ import time
 import types
 import uuid
 import warnings
+import weakref
 import zlib
 from abc import ABC
 from contextlib import contextmanager
+from types import ModuleType, TracebackType
 from typing import (
     Any,
-    List,
+    Callable,
     Dict,
+    List,
+    Mapping,
     NamedTuple,
+    Optional,
     Set,
     Tuple,
     Type,
     Union,
-    Callable,
-    Optional,
-    Mapping,
 )
-from types import TracebackType
 from urllib.parse import urlparse
 
 import cloudpickle as pickle
 import numpy as np
 import pandas as pd
-
-from .constants import MARS_LOG_PATH_KEY
-from ._utils import (  # noqa: F401 # pylint: disable=unused-import
+from xoscar._utils import (
+    TypeDispatcher,
+    new_random_id,
+    reset_id_random_seed,
     to_binary,
     to_str,
-    to_text,
+)
+
+from ._utils import (  # noqa: F401 # pylint: disable=unused-import
     NamedType,
-    TypeDispatcher,
+    Timer,
+    ceildiv,
+    register_tokenizer,
+    to_text,
     tokenize,
     tokenize_int,
-    register_tokenizer,
-    ceildiv,
-    reset_id_random_seed,
-    new_random_id,
-    Timer,
 )
 from .lib.version import parse as parse_version
-from .typing import ChunkType, TileableType, EntityType, OperandType
+from .typing import ChunkType, EntityType, OperandType, TileableType
 
 logger = logging.getLogger(__name__)
 random.seed(int(time.time()) * os.getpid())
@@ -116,8 +116,8 @@ if sys.platform.startswith("win"):
 
 
 try:
-    from pandas._libs.lib import NoDefault, no_default
     from pandas._libs import lib as _pd__libs_lib
+    from pandas._libs.lib import NoDefault, no_default
 
     _raw__reduce__ = type(NoDefault).__reduce__
 
@@ -356,44 +356,48 @@ class classproperty:
         return self.f(owner)
 
 
+class LazyModule:
+    def __init__(self, name, rename, globals_, locals_):
+        self.name = name
+        self._on_loads = []
+        self._rename = rename
+        self._globals = globals_
+        self._locals = locals_
+
+    def __getattr__(self, item):
+        if item.startswith("_pytest") or item in ("__bases__", "__test__"):
+            raise AttributeError(item)
+
+        real_mod = importlib.import_module(self.name)
+        if self._rename in self._globals:
+            self._globals[self._rename] = real_mod
+        elif self._locals is not None:
+            self._locals[self._rename] = real_mod
+        ret = getattr(real_mod, item)
+        for on_load_func in self._on_loads:
+            on_load_func()
+        # make sure on_load hooks only executed once
+        self._on_loads = []
+        return ret
+
+    def add_load_handler(self, func: Callable):
+        self._on_loads.append(func)
+        return func
+
+
 def lazy_import(
     name: str,
-    package: str = None,
-    globals: Dict = None,  # pylint: disable=redefined-builtin
-    locals: Dict = None,  # pylint: disable=redefined-builtin
+    globals_: Dict = None,  # pylint: disable=redefined-builtin
+    locals_: Dict = None,  # pylint: disable=redefined-builtin
     rename: str = None,
     placeholder: bool = False,
 ):
     rename = rename or name
     prefix_name = name.split(".", 1)[0]
-    globals = globals or inspect.currentframe().f_back.f_globals
-
-    class LazyModule:
-        def __init__(self):
-            self._on_loads = []
-
-        def __getattr__(self, item):
-            if item.startswith("_pytest") or item in ("__bases__", "__test__"):
-                raise AttributeError(item)
-
-            real_mod = importlib.import_module(name, package=package)
-            if rename in globals:
-                globals[rename] = real_mod
-            elif locals is not None:
-                locals[rename] = real_mod
-            ret = getattr(real_mod, item)
-            for on_load_func in self._on_loads:
-                on_load_func()
-            # make sure on_load hooks only executed once
-            self._on_loads = []
-            return ret
-
-        def add_load_handler(self, func: Callable):
-            self._on_loads.append(func)
-            return func
+    globals_ = globals_ or inspect.currentframe().f_back.f_globals
 
     if pkgutil.find_loader(prefix_name) is not None:
-        return LazyModule()
+        return LazyModule(name, rename, globals_, locals_)
     elif placeholder:
         return ModulePlaceholder(prefix_name)
     else:
@@ -423,8 +427,20 @@ class ModulePlaceholder:
         self._raises()
 
 
+def is_same_module(
+    mod1: Optional[Union[ModuleType, LazyModule]],
+    mod2: Optional[Union[ModuleType, LazyModule]],
+) -> bool:
+    if mod1 is None or mod2 is None:
+        return False
+
+    name1 = mod1.name if isinstance(mod1, LazyModule) else mod1.__name__
+    name2 = mod2.name if isinstance(mod2, LazyModule) else mod2.__name__
+    return name1 == name2
+
+
 def serialize_serializable(serializable, compress: bool = False):
-    from .serialization import serialize
+    from xoscar.serialization import serialize
 
     bio = io.BytesIO()
     header, buffers = serialize(serializable)
@@ -443,7 +459,7 @@ def serialize_serializable(serializable, compress: bool = False):
 
 
 def deserialize_serializable(ser_serializable: bytes):
-    from .serialization import deserialize
+    from xoscar.serialization import deserialize
 
     bio = io.BytesIO(ser_serializable)
     s_header_length = struct.unpack("Q", bio.read(8))[0]
@@ -452,31 +468,6 @@ def deserialize_serializable(ser_serializable: bytes):
     return deserialize(header2, buffers2)
 
 
-def register_ray_serializer(obj_type, serializer=None, deserializer=None):
-    try:
-        import ray
-
-        try:
-            ray.register_custom_serializer(
-                obj_type, serializer=serializer, deserializer=deserializer
-            )
-        except AttributeError:  # ray >= 1.0
-            try:
-                from ray.worker import global_worker
-
-                global_worker.check_connected()
-                context = global_worker.get_serialization_context()
-                context.register_custom_serializer(
-                    obj_type, serializer=serializer, deserializer=deserializer
-                )
-            except AttributeError:  # ray >= 1.2.0
-                ray.util.register_serializer(
-                    obj_type, serializer=serializer, deserializer=deserializer
-                )
-    except ImportError:
-        pass
-
-
 cudf = lazy_import("cudf")
 
 
@@ -582,8 +573,7 @@ def estimate_pandas_size(
 def build_fetch_shuffle(
     chunk: ChunkType, n_reducers=None, shuffle_fetch_type=None
 ) -> ChunkType:
-    from .core.operand import ShuffleProxy
-    from .core.operand import ShuffleFetchType
+    from .core.operand import ShuffleFetchType, ShuffleProxy
 
     chunk_op = chunk.op
     assert isinstance(chunk_op, ShuffleProxy)
@@ -693,11 +683,11 @@ def merge_chunks(chunk_results: List[Tuple[Tuple[int], Any]]) -> Any:
     from sklearn.base import BaseEstimator
 
     from .dataframe.utils import (
+        concat_on_columns,
+        get_xdf,
         is_dataframe,
         is_index,
         is_series,
-        get_xdf,
-        concat_on_columns,
     )
     from .lib.groupby_wrapper import GroupByWrapper
     from .tensor.array_utils import get_array_module, is_array
@@ -745,6 +735,8 @@ def merge_chunks(chunk_results: List[Tuple[Tuple[int], Any]]) -> Any:
         df = pd.concat([r[1].obj for r in chunk_results], axis=0)
         if not isinstance(v.keys, list):
             keys = v.keys
+            if isinstance(v.keys, pd.Series):
+                keys = pd.concat([r[1].keys for r in chunk_results])
         else:
             keys = []
             for idx, k in enumerate(v.keys):
@@ -765,7 +757,6 @@ def merge_chunks(chunk_results: List[Tuple[Tuple[int], Any]]) -> Any:
             group_keys=v.group_keys,
             squeeze=v.squeeze,
             observed=v.observed,
-            mutated=v.mutated,
         )
         return grouped.groupby_obj
     elif isinstance(v, (str, bytes, memoryview, BaseEstimator)):
@@ -789,8 +780,8 @@ def merge_chunks(chunk_results: List[Tuple[Tuple[int], Any]]) -> Any:
 
 
 def merged_chunk_as_tileable_type(merged, tileable: TileableType):
-    from .tensor.core import TensorOrder
     from .tensor.array_utils import get_array_module
+    from .tensor.core import TensorOrder
 
     if hasattr(tileable, "order") and tileable.ndim > 0:
         module = get_array_module(merged)
@@ -1152,8 +1143,8 @@ def get_dtype(dtype: Union[np.dtype, pd.api.extensions.ExtensionDtype]):
 def calc_object_overhead(chunk: ChunkType, shape: Tuple[int]) -> int:
     from .dataframe.core import (
         DATAFRAME_CHUNK_TYPE,
-        SERIES_CHUNK_TYPE,
         INDEX_CHUNK_TYPE,
+        SERIES_CHUNK_TYPE,
     )
 
     if not shape or np.isnan(shape[0]) or getattr(chunk, "dtypes", None) is None:
@@ -1193,7 +1184,10 @@ def arrow_array_to_objects(
                         obj.iloc[:, i].to_numpy(), index=obj.index
                     )
                 else:
-                    result.iloc[:, i] = obj.iloc[:, i]
+                    if pd.__version__ >= "1.5.0":
+                        result.isetitem(i, obj.iloc[:, i])
+                    else:  # pragma: no cover
+                        result.iloc[:, i] = obj.iloc[:, i]
             obj = result
     elif isinstance(obj, pd.Series):
         if isinstance(obj.dtype, ArrowDtype):
@@ -1480,25 +1474,20 @@ def register_asyncio_task_timeout_detector(
                 "MARS_DEBUG_ASYNCIO_TASK_EXCLUDE_FILTERS", "mars/oscar"
             )
             task_exclude_filters = task_exclude_filters.split(";")
-        if sys.version_info[:2] < (3, 7):
-            logger.warning(
-                "asyncio tasks timeout detector is not supported under python %s",
-                sys.version,
-            )
-        else:
-            loop = asyncio.get_running_loop()
-            logger.info(
-                "Create asyncio tasks timeout detector with check_interval %s task_timeout_seconds %s "
-                "task_exclude_filters %s",
-                check_interval,
-                task_timeout_seconds,
-                task_exclude_filters,
-            )
-            return loop.create_task(
-                asyncio_task_timeout_detector(
-                    check_interval, task_timeout_seconds, task_exclude_filters
-                )
+
+        loop = asyncio.get_running_loop()
+        logger.info(
+            "Create asyncio tasks timeout detector with check_interval %s task_timeout_seconds %s "
+            "task_exclude_filters %s",
+            check_interval,
+            task_timeout_seconds,
+            task_exclude_filters,
+        )
+        return loop.create_task(
+            asyncio_task_timeout_detector(
+                check_interval, task_timeout_seconds, task_exclude_filters
             )
+        )
     else:
         return None
 
@@ -1749,39 +1738,12 @@ def aiotask_wrapper(_f=None, exit_if_exception=False):
 
 
 def is_ray_address(address: str) -> bool:
-    from .oscar.backends.ray.communication import RayServer
-
-    if urlparse(address).scheme == RayServer.scheme:
+    if urlparse(address).scheme == "ray":
         return True
     else:
         return False
 
 
-# TODO: clean_up_func, is_on_ray and restore_func functions may be
-# removed or refactored in the future to calculate func size
-# with more accuracy as well as address some serialization issues.
-def is_on_ray(ctx):
-    from .services.task.execution.ray.context import (
-        RayExecutionContext,
-        RayExecutionWorkerContext,
-    )
-
-    # There are three conditions
-    #   a. mars backend
-    #   b. ray backend(oscar), c. ray backend(dag)
-    # When a. or b. is selected, ctx is an instance of ThreadedServiceContext.
-    #   The main difference between them is whether worker address matches ray scheme.
-    #   To avoid duplicated checks, here we choose the first worker address.
-    # When c. is selected, ctx is an instance of RayExecutionContext or RayExecutionWorkerContext,
-    #   while get_worker_addresses method isn't currently implemented in RayExecutionWorkerContext.
-    try:
-        worker_addresses = ctx.get_worker_addresses()
-    except AttributeError:  # pragma: no cover
-        assert isinstance(ctx, RayExecutionWorkerContext)
-        return True
-    return isinstance(ctx, RayExecutionContext) or is_ray_address(worker_addresses[0])
-
-
 def cache_tileables(*tileables):
     from .core import ENTITY_TYPE
 
@@ -1881,14 +1843,7 @@ def retry_callable(
     return retry_call
 
 
-def clean_mars_tmp_dir():
-    # clean Mars log file and Mars tmp dir
-    filename = os.environ.get(MARS_LOG_PATH_KEY)
-    if filename is not None:
-        os.environ.pop(MARS_LOG_PATH_KEY)
-        if os.path.exists(filename):
-            mars_tmp_dir = os.path.dirname(filename)
-            if os.path.exists(mars_tmp_dir):
-                # on windows platform, raise Permission Error
-                _windows: bool = sys.platform.startswith("win")
-                shutil.rmtree(mars_tmp_dir, ignore_errors=_windows)
+def random_string_and_digits(length: int) -> str:
+    return "".join(
+        random.choice(string.ascii_letters + string.digits) for _ in range(length)
+    )
