@@ -24,12 +24,15 @@ from ..._mars.lib.aio import new_isolation, stop_isolation
 from ..._mars.services.cluster.api import WebClusterAPI
 from ..._mars.session import new_session
 from ..._mars.utils import calc_size_by_str
+from ...compat._constants import SUPPORTED_PY_VERSIONS
+from ...utils import get_local_py_version
 from .config import (
     IngressConfig,
     NamespaceConfig,
     RoleBindingConfig,
     RoleConfig,
     ServiceConfig,
+    XorbitsReplicationConfig,
     XorbitsSupervisorsConfig,
     XorbitsWorkersConfig,
 )
@@ -116,6 +119,14 @@ class KubernetesCluster:
             raise TypeError("`pip` must be str or List[str] type.")
         if conda is not None and not isinstance(conda, (str, list)):  # pragma: no cover
             raise TypeError("`conda` must be str or List[str] type.")
+        local_py_version = get_local_py_version()
+        if (
+            image is None and local_py_version not in SUPPORTED_PY_VERSIONS
+        ):  # pragma: no cover
+            raise RuntimeError(
+                f"Xorbits does not support Kubernetes deployment under your python version {local_py_version}. "
+                f"Please change the python version to one of the following versions: {SUPPORTED_PY_VERSIONS}."
+            )
 
         self._api_client = kube_api_client
         self._core_api = kube_client.CoreV1Api(kube_api_client)
@@ -197,6 +208,9 @@ class KubernetesCluster:
         )
         self._pip = pip
         self._conda = conda
+        self._readiness_service_name = kwargs.pop(
+            "readiness_service_name", "xorbits-readiness-service"
+        )
 
     @property
     def namespace(self) -> Optional[str]:
@@ -254,6 +268,22 @@ class KubernetesCluster:
             self._namespace, service_config.build()
         )
 
+    def _create_readiness_service(self):
+        """
+        Start a simple ClusterIP service for the workers to detect whether the supervisor is successfully started.
+        """
+        readiness_service_config = ServiceConfig(
+            name=self._readiness_service_name,
+            service_type=None,
+            selector={"xorbits/service-type": XorbitsSupervisorsConfig.rc_name},
+            port=XorbitsReplicationConfig.default_readiness_port,
+            target_port=XorbitsReplicationConfig.default_readiness_port,
+            protocol="TCP",
+        )
+        self._core_api.create_namespaced_service(
+            self._namespace, readiness_service_config.build()
+        )
+
     def _get_ready_pod_count(self, label_selector: str) -> int:  # pragma: no cover
         query = self._core_api.list_namespaced_pod(
             namespace=self._namespace, label_selector=label_selector
@@ -336,6 +366,7 @@ class KubernetesCluster:
             use_local_image=self._use_local_image,
             pip=self._pip,
             conda=self._conda,
+            readiness_service_name=self._readiness_service_name,
         )
         workers_config.add_simple_envs(self._worker_extra_env)
         workers_config.add_labels(self._worker_extra_labels)
@@ -403,6 +434,7 @@ class KubernetesCluster:
             start_time = time.time()
             while True:
                 try:
+                    time.sleep(1)
                     # TODO use Xorbits API instead of mars.
                     cluster_api = WebClusterAPI(self._external_web_endpoint)
                     supervisors = await cluster_api.get_supervisors()
@@ -437,6 +469,7 @@ class KubernetesCluster:
 
             self._create_services()
             self._create_kube_service()
+            self._create_readiness_service()
 
             self._wait_services_ready()
 
