@@ -21,13 +21,15 @@ from typing import Any, Dict, List, Optional, Union
 
 from ... import __version__
 from ..._mars.utils import calc_size_by_str, parse_readable_size
+from ...compat._constants import COMPATIBLE_DEPS
+from ...utils import get_local_package_version, get_local_py_version
+from ._constants import SERVICE_PID_FILE
 
 try:
     from kubernetes.client import ApiClient
 except ImportError:  # pragma: no cover
     ApiClient = None
 
-DEFAULT_IMAGE = "xprobe/xorbits:v" + __version__
 DEFAULT_WORKER_CACHE_MEM = "40%"
 
 
@@ -94,7 +96,12 @@ class RoleConfig(KubeConfig):
                     "apiGroups": self._api_groups,
                     "resources": self._resources,
                     "verbs": self._verbs,
-                }
+                },
+                {
+                    "apiGroups": ["apps"],
+                    "resources": ["deployments", "deployments/scale"],
+                    "verbs": ["get", "watch", "list", "patch"],
+                },
             ],
         }
 
@@ -161,7 +168,7 @@ class ServiceConfig(KubeConfig):
     def __init__(
         self,
         name: str,
-        service_type: str,
+        service_type: Optional[str],
         selector: Union[str, Dict],
         port: int,
         target_port: Optional[int] = None,
@@ -404,7 +411,7 @@ class ProbeConfig:
     def __init__(
         self,
         initial_delay: int = 5,
-        period: int = 1,
+        period: int = 5,
         timeout: Optional[int] = None,
         success_thresh: Optional[int] = None,
         failure_thresh: Optional[int] = None,
@@ -442,6 +449,21 @@ class TcpSocketProbeConfig(ProbeConfig):
         return ret
 
 
+class CommandExecProbeConfig(ProbeConfig):
+    """
+    Configuration builder for command probe
+    """
+
+    def __init__(self, commands: List[str], **kwargs):
+        super().__init__(**kwargs)
+        self._commands = commands
+
+    def build(self):
+        ret = super().build()
+        ret["exec"] = {"command": self._commands}
+        return ret
+
+
 class ReplicationConfig(KubeConfig):
     """
     Base configuration builder for Kubernetes replication controllers
@@ -457,7 +479,7 @@ class ReplicationConfig(KubeConfig):
         resource_request: Optional["ResourceConfig"] = None,
         resource_limit: Optional["ResourceConfig"] = None,
         liveness_probe: Optional["ProbeConfig"] = None,
-        readiness_probe: Optional["ProbeConfig"] = None,
+        startup_probe: Optional["ProbeConfig"] = None,
         pre_stop_command: Optional[List[str]] = None,
         kind: Optional[str] = None,
         **kwargs,
@@ -477,7 +499,7 @@ class ReplicationConfig(KubeConfig):
         self._resource_limit = resource_limit
 
         self._liveness_probe = liveness_probe
-        self._readiness_probe = readiness_probe
+        self._startup_probe = startup_probe
 
         self._pre_stop_command = pre_stop_command
         self._use_local_image = kwargs.pop("use_local_image", False)
@@ -510,6 +532,9 @@ class ReplicationConfig(KubeConfig):
     def add_volume(self, vol):
         self._volumes.append(vol)
 
+    def config_init_containers(self) -> List[Dict]:
+        return []
+
     @staticmethod
     def get_install_content(source: Union[str, List[str]]) -> str:
         if isinstance(source, str):
@@ -521,6 +546,15 @@ class ReplicationConfig(KubeConfig):
             content = "\n".join(source)
         return content
 
+    @staticmethod
+    def get_compatible_packages() -> List[str]:
+        deps = []
+        for dep in COMPATIBLE_DEPS:
+            version = get_local_package_version(dep)
+            if version is not None:
+                deps.append(f"{dep}=={version}")
+        return deps
+
     @abc.abstractmethod
     def build_container_command(self) -> List:
         """Output container command"""
@@ -528,6 +562,14 @@ class ReplicationConfig(KubeConfig):
         # All install contents must be wrapped in double quotes to ensure the correctness
         # when passing to the shell script.
         # At the same time, each command is followed by a semicolon to separate the individual commands.
+        deps = self.get_compatible_packages()
+        if deps:
+            # Install for consistent packages must be at the top of all commands.
+            # This does not affect user behavior.
+            cmd += (
+                f'/srv/install.sh pip_compatible "{self.get_install_content(deps)}" ; '
+            )
+
         if self._pip is not None:
             cmd += f'/srv/install.sh pip "{self.get_install_content(self._pip)}" ; '
         if self._conda is not None:
@@ -569,8 +611,8 @@ class ReplicationConfig(KubeConfig):
                 "livenessProbe": self._liveness_probe.build()
                 if self._liveness_probe
                 else None,
-                "readinessProbe": self._readiness_probe.build()
-                if self._readiness_probe
+                "startupProbe": self._startup_probe.build()
+                if self._startup_probe
                 else None,
                 "lifecycle": lifecycle_dict or None,
                 "imagePullPolicy": "Never" if self._use_local_image else None,
@@ -580,6 +622,7 @@ class ReplicationConfig(KubeConfig):
     def build_template_spec(self) -> Dict:
         result = {
             "containers": [self.build_container()],
+            "initContainers": self.config_init_containers(),
             "volumes": [vol.build() for vol in self._volumes],
         }
         return dict((k, v) for k, v in result.items() if v)
@@ -647,11 +690,12 @@ class XorbitsReplicationConfig(ReplicationConfig, abc.ABC):
 
         super().__init__(
             self.rc_name,
-            image or DEFAULT_IMAGE,
+            image or self.get_default_image(),
             replicas,
             resource_request=req_res,
             resource_limit=limit_res if limit_resources else None,
-            readiness_probe=self.config_readiness_probe(),
+            liveness_probe=self.config_liveness_probe(),
+            startup_probe=self.config_startup_probe(),
             **kwargs,
         )
         if service_port:
@@ -662,6 +706,12 @@ class XorbitsReplicationConfig(ReplicationConfig, abc.ABC):
 
         self.add_labels({"xorbits/service-type": self.rc_name})
 
+    @staticmethod
+    def get_default_image():
+        image = "xprobe/xorbits:v" + __version__
+        image += f"-py{get_local_py_version()}"
+        return image
+
     def add_default_envs(self):
         self.add_env("MARS_K8S_POD_NAME", field_path="metadata.name")
         self.add_env("MARS_K8S_POD_NAMESPACE", field_path="metadata.namespace")
@@ -686,9 +736,26 @@ class XorbitsReplicationConfig(ReplicationConfig, abc.ABC):
         if self._modules:
             self.add_env("MARS_LOAD_MODULES", ",".join(self._modules))
 
-    def config_readiness_probe(self):
+    def config_liveness_probe(self):
+        """
+        Liveness probe works after the startup probe.
+        If the startup probe exists, the initial_delay of the liveness probe can be smaller.
+        """
         raise NotImplementedError  # pragma: no cover
 
+    @staticmethod
+    def config_startup_probe() -> "ProbeConfig":
+        """
+        The startup probe is used to check whether the startup is smooth.
+        The initial_delay of the startup probe can be sensitive to the system.
+        """
+        return CommandExecProbeConfig(
+            ["sh", "-c", f'until [ -f "{SERVICE_PID_FILE}" ]; do sleep 3s; done'],
+            failure_thresh=5,
+            initial_delay=10,
+            timeout=300,
+        )
+
     @staticmethod
     def get_local_app_module(mod_name) -> str:
         return __name__.rsplit(".", 1)[0] + "." + mod_name
@@ -718,8 +785,10 @@ class XorbitsSupervisorsConfig(XorbitsReplicationConfig):
         if self._web_port:
             self.add_port(self._web_port)
 
-    def config_readiness_probe(self) -> "TcpSocketProbeConfig":
-        return TcpSocketProbeConfig(self._readiness_port, timeout=60, failure_thresh=10)
+    def config_liveness_probe(self) -> "TcpSocketProbeConfig":
+        return TcpSocketProbeConfig(
+            self._readiness_port, timeout=60, failure_thresh=10, initial_delay=0
+        )
 
     def build_container_command(self):
         cmd = super().build_container_command()
@@ -754,6 +823,7 @@ class XorbitsWorkersConfig(XorbitsReplicationConfig):
         )
         min_cache_mem = kwargs.pop("min_cache_mem", None)
         self._readiness_port = kwargs.pop("readiness_port", self.default_readiness_port)
+        self._readiness_service_name = kwargs.pop("readiness_service_name", None)
         supervisor_web_port = kwargs.pop("supervisor_web_port", None)
 
         super().__init__(*args, **kwargs)
@@ -788,8 +858,29 @@ class XorbitsWorkersConfig(XorbitsReplicationConfig):
         if supervisor_web_port:
             self.add_env("MARS_K8S_SUPERVISOR_WEB_PORT", supervisor_web_port)
 
-    def config_readiness_probe(self) -> "TcpSocketProbeConfig":
-        return TcpSocketProbeConfig(self._readiness_port, timeout=60, failure_thresh=10)
+    def config_liveness_probe(self) -> "TcpSocketProbeConfig":
+        return TcpSocketProbeConfig(
+            self._readiness_port, timeout=60, failure_thresh=10, initial_delay=0
+        )
+
+    def config_init_containers(self) -> List[Dict]:
+        """
+        The worker pod checks whether the readiness port of the supervisor is successfully opened in the init container.
+        The worker pod will not start until the init container is executed successfully.
+        This ensures that the worker must start after the supervisor.
+        The image and command used by the init container refer to the Kubernetes official documentation (https://kubernetes.io/docs/concepts/workloads/pods/init-containers/).
+        """
+        return [
+            {
+                "name": "waiting-for-supervisors",
+                "image": "busybox:1.28",
+                "command": [
+                    "sh",
+                    "-c",
+                    f"until nslookup {self._readiness_service_name}.$(cat /var/run/secrets/kubernetes.io/serviceaccount/namespace).svc.cluster.local; do echo waiting for supervisors; sleep 5; done",
+                ],
+            }
+        ]
 
     def build_container_command(self):
         cmd = super().build_container_command()
