@@ -13,17 +13,37 @@
 # limitations under the License.
 
 
+import inspect
+from typing import Any, Callable, Dict, Optional
+
+from ...core.utils.fallback import unimplemented_func
+
+
 def __dir__():
     from ..mars_adapters import MARS_TENSOR_FFT_CALLABLES
+    from ..numpy_adapters.core import NUMPY_FFT_MEMBERS
 
-    return list(MARS_TENSOR_FFT_CALLABLES.keys())
+    global NUMPY_FFT_METHODS
+    import numpy
+
+    return list(MARS_TENSOR_FFT_CALLABLES.keys()) + list(NUMPY_FFT_MEMBERS.keys())
 
 
 def __getattr__(name: str):
     from ..mars_adapters import MARS_TENSOR_FFT_CALLABLES
+    from ..numpy_adapters.core import NUMPY_FFT_MEMBERS
 
     if name in MARS_TENSOR_FFT_CALLABLES:
         return MARS_TENSOR_FFT_CALLABLES[name]
-    else:
-        # TODO: fallback to numpy
-        raise AttributeError(name)
+    else:  # pragma: no cover
+        import numpy
+
+        if not hasattr(numpy.fft, name):
+            raise AttributeError(name)
+        elif name in NUMPY_FFT_MEMBERS:
+            return NUMPY_FFT_MEMBERS[name]
+        else:
+            if inspect.ismethod(getattr(numpy.fft, name)):
+                return unimplemented_func
+            else:
+                raise AttributeError(name)
