@@ -12,18 +12,33 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import inspect
+
+from ...core.utils.fallback import unimplemented_func
+
 
 def __dir__():
     from ..mars_adapters import MARS_TENSOR_LINALG_CALLABLES
+    from ..numpy_adapters.core import NUMPY_LINALG_MEMBERS
 
-    return list(MARS_TENSOR_LINALG_CALLABLES.keys())
+    return list(MARS_TENSOR_LINALG_CALLABLES.keys()) + list(NUMPY_LINALG_MEMBERS.keys())
 
 
 def __getattr__(name: str):
     from ..mars_adapters import MARS_TENSOR_LINALG_CALLABLES
+    from ..numpy_adapters.core import NUMPY_LINALG_MEMBERS
 
     if name in MARS_TENSOR_LINALG_CALLABLES:
         return MARS_TENSOR_LINALG_CALLABLES[name]
     else:
-        # TODO: fallback to numpy
-        raise AttributeError(name)
+        import numpy
+
+        if not hasattr(numpy.linalg, name):
+            raise AttributeError(name)
+        elif name in NUMPY_LINALG_MEMBERS:
+            return NUMPY_LINALG_MEMBERS[name]
+        else:  # pragma: no cover
+            if inspect.ismethod(getattr(numpy.linalg, name)):
+                return unimplemented_func
+            else:
+                raise AttributeError(name)
