@@ -11,19 +11,33 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+import inspect
+
+from ...core.utils.fallback import unimplemented_func
 
 
 def __dir__():
     from ..mars_adapters import MARS_TENSOR_RANDOM_CALLABLES
+    from ..numpy_adapters.core import NUMPY_RANDOM_MEMBERS
 
-    return list(MARS_TENSOR_RANDOM_CALLABLES.keys())
+    return list(MARS_TENSOR_RANDOM_CALLABLES.keys()) + list(NUMPY_RANDOM_MEMBERS.keys())
 
 
 def __getattr__(name: str):
     from ..mars_adapters import MARS_TENSOR_RANDOM_CALLABLES
+    from ..numpy_adapters.core import NUMPY_RANDOM_MEMBERS
 
     if name in MARS_TENSOR_RANDOM_CALLABLES:
         return MARS_TENSOR_RANDOM_CALLABLES[name]
     else:
-        # TODO: fallback to numpy
-        raise AttributeError(name)
+        import numpy
+
+        if not hasattr(numpy.random, name):
+            raise AttributeError(name)
+        elif name in NUMPY_RANDOM_MEMBERS:
+            return NUMPY_RANDOM_MEMBERS[name]
+        else:  # pragma: no cover
+            if inspect.ismethod(getattr(numpy.random, name)):
+                return unimplemented_func
+            else:
+                raise AttributeError(name)
